# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
#

from zengine.forms import JsonForm, fields
from zengine.views.crud import CrudView
from collections import OrderedDict
from ulakbus.services.zato_wrapper import DersProgramiOlustur
from ulakbus.services.zato_wrapper import SinavProgramiOlustur
from ulakbus.models import Room, Okutman, DersEtkinligi, Donem, DerslikZamanPlani, SinavEtkinligi, Sube
from zengine.lib.translation import gettext_lazy as __, gettext as _, format_time, format_datetime
from datetime import time

ARAMA_TURU = [
    (1, 'Derslik'),
    (2, 'Öğretim Elemanı')
]

MESSAGE = {"kayit_var": {"type": 'info',
                         "title": __(u'Yayınlanmış Program Var!'),
                         "msg": __(u'Yayınlanan programınız bulunmaktadır. Tekrardan program oluşturamazsınız.')},
           "hatasiz_sonuc": {"type": 'info',
                             "title": __(u'Yayınlanmamış Program Var!'),
                             "msg": __(u'Yayınlanmayan programınızı inceleyip yayınlayabilirsiniz.')},
           "hatali_sonuc": {"type": 'warning',
                            "title": __(u'Hatalı Sonuçlar Var!'),
                            "msg": __(u'Oluşturulan programda hatalı sonuçlar bulunmaktadır. Lütfen tekrardan'
                                      u'verileri düzenleyip çalıştırınız.')},
           "basarili": {"type": 'info',
                        "title": __(u'Program Başarıyla Oluşturuldu!'),
                        "msg": __(u'Yayınlanmayan programınızı inceleyip yayınlayabilirsiniz.')},
           "yayinlandi": {"type": 'info',
                          "title": __(u'Program Yayınlandı!'),
                          "msg": __(u'Oluşturulan Program Başarıyla Yayınlandı')}}


class AramaForm(JsonForm):
    class Meta:
        title = __(u'Öğretim Elemanı veya Derslik Ara')

    arama_sec = fields.String(__(u'Arama Seçeneği'), choices=ARAMA_TURU, default=1)
    arama_text = fields.String(__(u" "), required=False)
    arama_button = fields.Button(__(u'Ara'))
    vazgec_button = fields.Button(__(u'Geri'), cmd='vazgec')


class ProgramOlustur(CrudView):

    def calculate(self, obj):
        """

        Args:
            obj: Ders veya Sınav etkinliği modeli

        Returns: model_object.count() >> bölüm ve güncel doneme ait model sayısı,
                 solved_count >> çözülen model sayısı,
                 published_count >> yayınlanan model sayısı

        """
        model_object = obj.objects.filter(bolum=self.current.role.unit, donem=Donem.guncel_donem(self.current))

        solved_count = model_object.filter(solved=True).count()

        published_count = model_object.filter(published=True).count()

        return model_object.count(), solved_count, published_count

    def kontrol(self, obj_count, solved, published):
        """
            Yayınlanan veya çözülmüş ama yayınlanmamış program var mı kontrolü yapılır.
        """
        self.current.task_data['published'] = published
        self.current.task_data['solved'] = solved
        self.current.task_data['obj_count'] = obj_count

    def sinav_kontrol(self):
        se_count, solved_count, published_count = self.calculate(SinavEtkinligi)
        self.kontrol(se_count, solved_count, published_count)

    def ders_kontrol(self):
        de_count, solved_count, published_count = self.calculate(DersEtkinligi)
        self.kontrol(de_count, solved_count, published_count)

    def program_hesaplama_baslat(self):
        _form = JsonForm(title=_(u"Program Oluştur"))
        _form.button = fields.Button(_(u'Başlat'))
        self.form_out(_form)

    def program_hesapla(self, obj):
        """

        Args:
            obj: ZatoService objesi

        Zato service çağırılıp hesaplama başlatılır. Dönen response, hesaplamanın başladığını yada
        başlamadığını bildirir.

        """
        response = obj.zato_request()

        if response:
            msg = {"type": 'info',
                   "title": _(u'Program Oluşturuluyor'),
                   "msg": _(u'Program için yaptığınız taleb başarıyla alınmıştır.')}
            self.current.task_data['LANE_CHANGE_MSG'] = msg
        else:
            msg = {"type": 'warning',
                   "title": _(u'Sistemde Sorun Oluştu'),
                   "msg": _(u'Lütfen tekrardan programı oluşturmayı çalıştırnız!')}
            self.current.task_data['LANE_CHANGE_MSG'] = msg

    def sinav_programi_hesapla(self):
        sp = SinavProgramiOlustur(service_payload={"bolum": self.current.role.unit.yoksis_no,
                                                   "kullanici": self.current.user.key,
                                                   "sinav_turleri": [1],
                                                   "url": self.current.get_wf_link()})
        self.program_hesapla(sp)

    def ders_programi_hesapla(self):
        dp = DersProgramiOlustur(service_payload={"bolum": self.current.role.unit.yoksis_no,
                                                  "kullanici": self.current.user.key,
                                                  "url": self.current.get_wf_link()})

        self.program_hesapla(dp)

    def servis_bilgi_mesaji(self):
        """
            Kullanıcıyı programının başladığı yada başlamadığına dair bilgilendirme mesajı gösterir
        """
        self.current.output['msgbox'] = self.current.task_data['LANE_CHANGE_MSG']

    def sinav_programi_sonucu(self):
        se_count, solved_count, published_count = self.calculate(SinavEtkinligi)
        self.current.task_data['solved_count'] = solved_count
        self.current.task_data['obj_count'] = se_count

    def ders_programi_sonucu(self):
        de_count, solved_count, published_count = self.calculate(DersEtkinligi)
        self.current.task_data['solved_count'] = solved_count
        self.current.task_data['obj_count'] = de_count

    def yayinlanmamis_kayit(self):
        _form = JsonForm(title=_(u"İşleme devam etmek için tıklayınız!"))
        _form.devam = fields.Button(_(u'Devam'))
        self.form_out(_form)
        self.current.output['msgbox'] = MESSAGE['hatasiz_sonuc']

    def hatasiz(self):
        """
            Hatasız oluşan programı inceleyebilir yada yayınlanır.
        """
        _form = JsonForm(title=_(u"Programı İncele Veya Yayınla"))
        _form.incele = fields.Button(_(u"İncele"), cmd='incele')
        _form.yayinla = fields.Button(_(u"Yayınla"), cmd='bitir')
        self.form_out(_form)

    def hatali(self):
        self.current.task_data['LANE_CHANGE_MSG'] = MESSAGE['hatali_sonuc']

    def derslik_ogretim_elemani_ara(self):
        """
            İncelediğiniz programda Öğretim Elemanlarına yada Dersliklere bakılır.
        """
        self.form_out(AramaForm(self.object, current=self.current))

    def arama_secim(self):
        self.current.task_data['arama'] = self.input['form']['arama_sec']

    def derslik_arama(self):
        text = str(self.input['form']['arama_text'])
        rooms = [r for r in Room.objects.search_on('code',
                                                   contains=text,
                                                   unit=self.current.role.unit)
                 if self.input['wf'] == 'ders_programi_hazirla'
                 or self.input['wf'] == 'sinav_programi_hazirla']

        if len(rooms) > 1:
            self.current.task_data['arama_text'] = text
            self.current.task_data['sonuc'] = 2
        elif len(rooms) == 1:
            self.current.task_data['data_key'] = rooms[0].key
            self.current.task_data['sonuc'] = 1
        else:
            self.current.task_data['sonuc'] = 0

    def ogretim_elemani_arama(self):
        text = str(self.input['form']['arama_text'])
        okutmanlar = [ok for ok in Okutman.objects.search_on('ad', 'soyad',
                                                             startswith=text.split()[0],
                                                             endswith=text.split()[-1],
                                                             unit=self.current.role.unit)
                      if self.input['wf'] == 'ders_programi_hazirla'
                      or self.input['wf'] == 'sinav_programi_hazirla']

        if len(okutmanlar) > 1:
            self.current.task_data['arama_text'] = text
            self.current.task_data['sonuc'] = 2
        elif len(okutmanlar) == 1:
            self.current.task_data['data_key'] = okutmanlar[0].key
            self.current.task_data['sonuc'] = 1
        else:
            self.current.task_data['sonuc'] = 0

    def coklu_sonuc(self):
        text = str(self.current.task_data['arama_text'])
        if self.current.task_data['arama'] == 2:
            self.output['objects'] = [[_(u'Ad'), _(u'Soyad')]]
            derslik = False
            datas = [ok for ok in Okutman.objects.search_on('ad', 'soyad',
                                                            startswith=text.split()[0],
                                                            endswith=text.split()[-1],
                                                            unit=self.current.role.unit) if ok.ders_etkinligi_set]
        else:
            derslik = True
            self.output['objects'] = [[_(u'Code'), _(u'Name')]]
            datas = [r for r in Room.objects.search_on('code', contains=text,
                                                       unit=self.current.role.unit) if r.ders_etkinligi_set]
        for data in datas:
            data_list = OrderedDict({})
            if not derslik:
                data_list[_(u'Ad')] = data.ad
                data_list[_(u'Soyad')] = data.soyad
            else:
                data_list[_(u'Code')] = data.code
                data_list[_(u'Name')] = data.name
            item = {
                'type': "table-multiRow",
                'fields': data_list,
                'actions': [
                    {'name': _(u'Goster'), 'cmd': 'tek_sonuc', 'show_as': 'button',
                     'object_key': 'data_key'}
                ],
                'key': data.key
            }
            self.output['objects'].append(item)

    def ders_detay_goster(self):
        if 'data_key' in self.current.input:
            obj_key = self.current.input['data_key']
        else:
            obj_key = self.current.task_data['data_key']
        if self.current.task_data['arama'] == 1:
            data_etkinlik = DersEtkinligi.objects.filter(room_id=obj_key)
            obj = Room.objects.get(obj_key)

        else:
            data_etkinlik = DersEtkinligi.objects.filter(okutman_id=obj_key)
            obj = Okutman.objects.get(obj_key)

        days = [_(u"Pazartesi"), _(u"Salı"), _(u"Çarşamba"), _(u"Perşembe"), _(u"Cuma"), _(u"Cumartesi"), _(u"Pazar")]
        self.output['objects'] = [days]

        def etkinlik(de):
            """
            Ders etkinligi formatlar ve dondurur.

            :param de: ders etkinligi
            :return: ders adi ve zamani
            """
            aralik = "{baslangic} - {bitis}".format(
                                        baslangic=format_time(time(int(de.baslangic_saat), int(de.baslangic_dakika))),
                                        bitis=format_time(time(int(de.bitis_saat), int(de.bitis_dakika))))

            return "\n\n**%s**\n%s\n\n" % (aralik, de.ders.ad)

        data_list = []
        for day in days:
            data_list.append(
                ''.join(["%s" % etkinlik(de) for de in data_etkinlik.filter(gun=days.index(day) + 1)]))

        self.detay_goster(data_list, obj)

    def sinav_detay_goster(self):
        obj_key = self.current.task_data['data_key']
        if self.input['form']['arama_sec'] == 1:
            sinav_etkinligi = SinavEtkinligi.objects.raw("sinav_yerleri.room_id:" + obj_key)
            obj = Room.objects.get(obj_key)

        else:
            sinav_etkinligi = map(lambda s: SinavEtkinligi.objects.get(sube=s), Sube.objects.filter(
                                                            okutman_id=obj_key, donem=Donem.guncel_donem(self.current)))
            obj = Okutman.objects.get(obj_key)

        days = [_(u"Pazartesi"), _(u"Salı"), _(u"Çarşamba"), _(u"Perşembe"), _(u"Cuma"), _(u"Cumartesi"), _(u"Pazar")]

        self.output['objects'] = [days]

        def etkinlik(de):
            """
            Ders etkinligi formatlar ve dondurur.

            :param de: ders etkinligi
            :return: ders adi ve zamani
            """
            aralik = format_datetime(de.tarih)
            return "\n\n**%s**\n%s\n\n" % (aralik, de.ders.ad)

        data_list = []
        for i, day in enumerate(days):
            data_list.append(
                ''.join(["%s" % etkinlik(de) for de in filter(lambda d: d.tarih.isoweekday() == i + 1,
                                                              sinav_etkinligi)]))
        self.detay_goster(data_list, obj)

    def detay_goster(self, data_list, obj):
        item = {
            "title": "%s - Detaylı Zaman Tablosu" % str(obj.__unicode__()),
            'type': "table-multiRow",
            'fields': data_list,
            "actions": False,
        }
        self.output['objects'].append(item)
        _json = JsonForm(title=obj.__unicode__() +_(u" Detaylı Zaman Tablosu"))
        _json.tamamla = fields.Button(_(u"Bitir"))
        if not self.current.task_data['sonuc'] == 1:
            _json.geri = fields.Button(_(u"Geri"), cmd='geri_coklu')
        self.form_out(_json)

    def sonuc_bulunamadi(self):
        msg = {
            'type': 'warning', "title": _(u'Kayıt Bulunamadı'),
            "msg": _(u'İlgili kayıt bulunamadı.')
        }
        self.current.output['msgbox'] = msg
        _form = JsonForm(title=_(u' '))
        _form.devam = fields.Button(_(u'Bitir'))
        self.form_out(_form)

    def kayit_var(self):
        self.current.output['msgbox'] = MESSAGE['kayit_var']

    def sinav_yayinla(self):
        se = SinavEtkinligi.objects.filter(bolum=self.current.role.unit, donem=Donem.guncel_donem(self.current))
        for s in se:
            s.published = True
            s.save()

    def ders_yayinla(self):
        de = DersEtkinligi.objects.filter(bolum=self.current.role.unit, donem=Donem.guncel_donem(self.current))
        for d in de:
            d.published = True
            d.save()

    def bilgilendirme(self):
        self.current.output['msgbox'] = MESSAGE['yayinlandi']



# -*-  coding: utf-8 -*-
"""Auth Modülü

Bu modül Ulakbüs uygulaması için authorization ile ilişkili data modellerini içerir.

"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
import hashlib

from pyoko import field
from pyoko import Model, ListNode
from pyoko import LinkProxy
from pyoko.conf import settings
from pyoko.lib.utils import lazy_property

from zengine.auth.permissions import get_all_permissions
from zengine.dispatch.dispatcher import receiver
from zengine.lib.decorators import role_getter

from zengine.signals import crud_post_save
from zengine.lib.cache import Cache
from zengine.lib.translation import gettext_lazy as _, gettext
from zengine.messaging.lib import BaseUser
from zengine.lib import translation

try:
    from zengine.lib.exceptions import PermissionDenied
except ImportError:
    class PermissionDenied(Exception):
        pass

from pyoko.exceptions import IntegrityError


class User(Model, BaseUser):
    """User modeli

    User modeli Ulakbus temel kullanıcı modelidir. Temel kullanıcı
    bilgilerini içerir. Ulakbus'de işlem yapan/yapılan her kullanıcıya
    ait bir ve tek kullanıcı olması zorunludur.

    """
    avatar = field.File(_(u"Profil Fotoğrafı"), random_name=True, required=False)
    username = field.String(_(u"Kullanıcı Adı"), index=True, unique=True)
    password = field.String(_(u"Parola"))
    e_mail = field.String(_(u"E-Posta"), index=True, unique=True)
    name = field.String(_(u"Ad"), index=True)
    surname = field.String(_(u"Soyad"), index=True)
    superuser = field.Boolean(_(u"Super user"), default=False)
    last_login_role_key = field.String(_(u"Son Giriş Yapılan Rol"))
    locale_language = field.String(
        _(u"Tercih Edilen Dil Formatı"),
        index=False,
        default=settings.DEFAULT_LANG,
        choices=translation.available_translations.items()
    )
    locale_datetime = field.String(_(u"Tercih Edilen Gün ve Zaman Formatı"), index=False,
                                   default=settings.DEFAULT_LOCALIZATION_FORMAT,
                                   choices=translation.available_datetimes.items())
    locale_number = field.String(_(u"Tercih Edilen Sayı Formatı"), index=False,
                                 default=settings.DEFAULT_LOCALIZATION_FORMAT,
                                 choices=translation.available_numbers.items())

    class Meta:
        app = 'Sistem'
        verbose_name = _(u"Kullanıcı")
        verbose_name_plural = _(u"Kullanıcılar")
        search_fields = ['username', 'name', 'surname']

    @lazy_property
    def full_name(self):
        return "%s %s" % (self.name, self.surname)

    def last_login_role(self):
        """
        Eğer kullanıcı rol geçişi yaparsa, kullanıcının last_login_role_key
        field'ına geçiş yaptığı rolün keyi yazılır. Kullanıcı çıkış yaptığında
         ve tekrardan giriş yaptığında son rolü bu field'dan öğrenilir. Eğer
        kullanıcının last_login_role_key field'ı dolu ise rol bilgisi oradan alınır.
        Yoksa kullanıcının role_set'inden default rolü alınır.

        """
        last_key = self.last_login_role_key
        return Role.objects.get(last_key) if last_key else self.role_set[0].role

    def pre_save(self):
        if not self.username or not self.password:
            raise IntegrityError
        self.encrypt_password()

    def post_creation(self):
        self.prepare_channels()

    def get_avatar_url(self):
        if self.avatar:
            return BaseUser.get_avatar_url(self)
        else:
            # FIXME: This is for fun, remove when we resolve static hosting problem
            return "https://www.gravatar.com/avatar/%s" % hashlib.md5(
                "%s@gmail.com" % self.username).hexdigest()

    def __unicode__(self):
        return "%s %s" % (self.name, self.surname)

class Permission(Model):
    """Permission modeli

    Kullanıcı yetkilerinin tanımlandığı bilgilerin bulunguğu modeldir.

    """
    name = field.String(_(u"İsim"), index=True)
    code = field.String(_(u"Kod Adı"), index=True)
    description = field.String(_(u"Tanım"), index=True)

    class Meta:
        app = 'Sistem'
        verbose_name = _(u"Yetki")
        verbose_name_plural = _(u"Yetkiler")
        list_fields = ["name", "code", "description"]
        search_fields = ["name", "code", "description"]

    def __unicode__(self):
        return "%s %s" % (self.name, self.code)

    def get_permitted_users(self):
        """
        Get users which has this permission

        Returns:
            User list
        """
        users = set()
        for ars in self.abstract_role_set.objects.filter():
            for r in ars.role_set.objects.filter():
                users.add(r.user)
        for r in self.role_set.objects.filter():
            users.add(r.user)
        return users

    def get_permitted_roles(self):
        """
        Get roles which has this permission

        Returns:
            Role list
        """
        roles = set()
        for ars in self.abstract_role_set.objects.filter():
            for r in ars.role_set.objects.filter():
                roles.add(r)
        for r in self.role_set.objects.filter():
            roles.add(r)
        return roles


class AbstractRole(Model):
    """AbstractRole Modeli

    Soyut Rol modeli yetkilerin gruplandığı temel role modelidir.

    """
    id = field.Integer(_(u"ID No"), index=True)
    name = field.String(_(u"İsim"), index=True)
    read_only = field.Boolean(_(u"Read Only"))

    class Meta:
        app = 'Sistem'
        verbose_name = _(u"Soyut Rol")
        verbose_name_plural = _(u"Soyut Roller")
        search_fields = ['id', 'name']

    def __unicode__(self):
        return "%s" % self.name

    def get_permissions(self):
        """
        Soyut role ait Permission nesnelerini bulur ve code değerlerini
        döner.

        Returns:
            list: Permission code değerleri

        """
        return [p.permission.code for p in self.Permissions if p.permission.code]

    def add_permission(self, perm):
        """
        Soyut Role Permission nesnesi tanımlamayı sağlar.

        Args:
            perm (object):

        """
        self.Permissions(permission=perm)
        PermissionCache.flush()
        self.save()

    def add_permission_by_name(self, code, save=False):
        """
        Soyut role Permission eklemek veya eklenebilecek Permission
        nesnelerini verilen ``code`` parametresine göre listelemek olmak
        üzere iki şekilde kullanılır.

        Args:
            code (str): Permission nesnelerini filtre etmekte kullanılır
            save (bool): True ise Permission ekler, False ise Permission
                listesi döner.

        Returns:
            list: ``save`` False ise Permission listesi döner.

        """
        if not save:
            return ["%s | %s" % (p.name, p.code) for p in
                    Permission.objects.filter(code__contains=code)]
        PermissionCache.flush()
        for p in Permission.objects.filter(code__contains=code):
            if p not in self.Permissions:
                self.Permissions(permission=p)
        if p:
            self.save()

    class Permissions(ListNode):
        permission = Permission()


class Unit(Model):
    """Unit modeli

    Kullanıcılara rol tanımlarken kullanılan, kullanıcının hangı birimde
    olduğunu taşıyan modeldir.
    Akademik ve idari birimlerin özelliklerini taşır.

    """
    name = field.String(_(u"İsim"), index=True)
    long_name = field.String(_(u"Uzun İsim"), index=True)
    yoksis_no = field.Integer(_(u"Yoksis ID"), index=True)
    unit_type = field.String(_(u"Birim Tipi"), index=True)
    parent_unit_no = field.Integer(_(u"Üst Birim ID"), index=True)
    current_situation = field.String(_(u"Guncel Durum"), index=True)
    language = field.String(_(u"Öğrenim Dili"), index=True)
    learning_type = field.String(_(u"Öğrenme Tipi"), index=True)
    osym_code = field.String(_(u"ÖSYM Kodu"), index=True)
    opening_date = field.Date(_(u"Açılış Tarihi"), index=True)
    learning_duration = field.Integer(_(u"Öğrenme Süresi"), index=True)
    english_name = field.String(_(u"İngilizce Birim Adı."), index=True)
    quota = field.Integer(_(u"Birim Kontenjan"), index=True)
    city_code = field.Integer(_(u"Şehir Kodu"), index=True)
    district_code = field.Integer(_(u"Semt Kodu"), index=True)
    unit_group = field.Integer(_(u"Birim Grup"), index=True)
    foet_code = field.Integer(_(u"FOET Kodu"), index=True)  # yoksis KILAVUZ_KODU mu?
    is_academic = field.Boolean(_(u"Akademik"))
    is_active = field.Boolean(_(u"Aktif"))
    uid = field.Integer(index=True)
    parent = LinkProxy('Unit', verbose_name=_(u'Üst Birim'), reverse_name='alt_birimler')
    # parent = field.String(verbose_name='Üst Birim') # fake

    @classmethod
    def get_role_keys(cls, unit_key):
        """recursively gets all roles (keys) under given unit"""
        return cls.get_role_keys_by_yoksis(Unit.objects.get(unit_key).yoksis_no)
        # stack = Role.objects.filter(unit_id=unit_key).values_list('user_id', flatten=True)
        # for unit_key in cls.objects.filter(parent_id=unit_key).values_list('key', flatten=True):
        #     stack.extend(cls.get_role_keys(unit_key))
        # return stack

    @classmethod
    def get_role_keys_by_yoksis(cls, yoksis_no):
        # because we don't refactor our data to use Unit.parent, yet!
        stack = Role.objects.filter(unit_id=Unit.objects.get(yoksis_no=yoksis_no).key).values_list('key', flatten=True)
        for yoksis_no in cls.objects.filter(parent_unit_no=yoksis_no).values_list('yoksis_no', flatten=True):
            stack.extend(cls.get_role_keys_by_yoksis(yoksis_no))
        return stack

    class Meta:
        app = 'Sistem'
        verbose_name = _(u"Unit")
        verbose_name_plural = _(u"Units")
        search_fields = ['name', 'yoksis_no']
        list_fields = ['name', 'unit_type']

    def __unicode__(self):
        return '%s - %s - %s' % (self.name, self.english_name, self.yoksis_no)


ROL_TIPI = [
    (1, _(u'Personel')),
    (2, _(u'Ogrenci')),
    (3, _(u'Harici'))
]


class PermissionCache(Cache):
    """PermissionCache sınıfı Kullanıcıya Permission nesnelerinin
    kontrolünü hızlandırmak için yetkileri cache bellekte saklamak ve
    gerektiğinde okumak için oluşturulmuştur.
    """
    PREFIX = 'PRM'

    def __init__(self, role_id):
        super(PermissionCache, self).__init__(role_id)


class Role(Model):
    """Role modeli
    Kullanıcıların rol ile ilişkilendiği modeldir. Kullanıcıların birden
    fazla rolü olabilir. Kullanıcıya atanmış Rol (Role) ve rolde
    tanımlanmış Soyut Rol (AbstractRole) nesnelerindeki yetkiler bir
    araya getirilerek kullanıcının yetkileri belirlenir.

    """
    abstract_role = AbstractRole()
    user = User()
    unit = Unit()
    typ = field.Integer(_(u"Rol Tipi"), choices=ROL_TIPI)
    name = field.String(_(u"Rol Adı"), hidden=True)

    class Meta:
        app = 'Sistem'
        verbose_name = _(u"Rol")
        verbose_name_plural = _(u"Roller")
        search_fields = ['name']
        list_fields = []
        crud_extra_actions = [{'name': _(u'İzinleri Düzenle'), 'wf': 'permissions', 'show_as': 'button'}]

    @property
    def is_staff(self):
        return self.typ == 1

    @property
    def is_student(self):
        return self.typ == 2

    def get_user(self):
        """
        Returns:
            object: user nesnesi
        """
        return self.user

    def __unicode__(self):
        return gettext(u"Role %s") % self.name or (self.key if self.is_in_db() else '')

    class Permissions(ListNode):
        permission = Permission()

        def __unicode__(self):
            return "%s" % self.permission

    def get_db_permissions(self):
        """
        Role nesnesinin AbstractRole özelliğine ait yetkileri getirir.

        Returns:
            list: yetki listesi

        """
        return [p.permission.code for p in self.Permissions if p.permission.code] + (
            self.abstract_role.get_permissions() if self.abstract_role.key else [])

    def _cache_permisisons(self, pcache):
        """
        Cache belleğe yetkileri yazar ve bu yetkileri döner.
        Args:
            pcache: PermissionCache nesnesi

        Returns:
            list: yetki listesi

        """
        perms = self.get_db_permissions()
        pcache.set(perms)
        return perms

    def get_permissions(self):
        """Bellekteki Permission nesnelerini döner.
        Returns:
            object:
        """
        pcache = PermissionCache(self.key)
        return pcache.get() or self._cache_permisisons(pcache)

    def add_permission(self, perm):
        """Yeni Permission ekler ve bellekteki Role nesnesine ait
        Permission nesnelerini siler.

        Args:
            perm: Permission nesnesi

        """
        self.Permissions(permission=perm)
        PermissionCache(self.key).delete()
        self.save()

    def remove_permission(self, perm):
        """
        Removes a :class:`Permission` from the role

        Args:
             perm: :class:`Permission` object.
        """
        del self.Permissions[perm.key]
        PermissionCache(self.key).delete()
        self.save()

    def add_permission_by_name(self, code, save=False):
        """
        Role nesnesine Permission eklemek veya eklenebilecek Permission
        nesnelerini verilen ``code`` parametresine göre listelemek olmak
        üzere iki şekilde kullanılır.

        Args:
            code (str): Permission nesnelerini filtre etmekte kullanılır.
            save (bool): True ise Permission ekler, False ise Permission
                listesi döner.

        Returns:
            list: ``save`` False ise Permission listesi döner.

        """
        if not save:
            return ["%s | %s" % (p.name, p.code) for p in
                    Permission.objects.filter(code__contains=code)]
        PermissionCache(self.key).delete()
        for p in Permission.objects.filter(code__contains=code):
            if p not in self.Permissions:
                self.Permissions(permission=p)
        if p:
            self.save()

    def _make_name(self):
        """
        Nesnenin soyut rolü veya bir kullanıcısı varsa soyut rol ismi ve
        kullanıcı username'ini birleştirerek bir name stringi oluşturur.
        Bu iki özellik yok ise Role key'ini kullanarak bir string değer
        üretir.
        Returns:
            str:

        """
        if self.abstract_role.key or self.user.key:
            return "%s | %s" % (self.abstract_role.name, self.user.username)
        else:
            return "Role #%s" % self.key if self.is_in_db() else ''

    def pre_save(self):
        """
        Kayıt edilmeden önce nesnenin adını ``_make_name()`` metodunu
        çağırarak oluşturur.
        """
        self.name = self._make_name()


    @classmethod
    @role_getter("Bölüm Başkanları")
    def get_bolum_baskanlari(cls):
        """fake"""
        return []

    def send_notification(self, title, message, typ=1, url=None, sender=None):
        """
        sends a message to user of this role's private mq exchange

        """
        self.user.send_notification(title=title, message=message, typ=typ, url=url, sender=sender)


class LimitedPermissions(Model):
    """LimitedPermissions modeli
    Bu modelde tutulan bilgilerle mevcut yetkilere sınırlandırmalar
    getirilir.

    - Başlangıç ve bitiş tarihine göre sınırlandırma uygulanan yetkiler
    o tarih aralığında geçerli olur.

    - Verilen IPList özelliğine göre bu IPList listesi içindeki
    ip'lerden gelen requestlere cevap verecek şekilde kısıtlanır.

    """
    restrictive = field.Boolean(_(u"Sınırlandırıcı"), default=False)
    time_start = field.String(_(u"Başlama Tarihi"), index=True)
    time_end = field.String(_(u"Bitiş Tarihi"), index=True)

    class Meta:
        app = 'Sistem'
        verbose_name = _(u"Sınırlandırılmış Yetki")
        verbose_name_plural = _(u"Sınırlandırılmış Yetkiler")

    def __unicode__(self):
        return "%s - %s" % (self.time_start, self.time_end)

    class IPList(ListNode):
        ip = field.String()

    class Permissions(ListNode):
        permission = Permission()

    class AbstractRoles(ListNode):
        abstract_role = AbstractRole()

    class Roles(ListNode):
        role = Role()


class AuthBackend(object):
    """AuthBackend sınıfı Auth modulüne ait işlemleri gerçekleştirmek
    için kullanılan bir dizi metodu içerir.

    """

    def __init__(self, current):
        self.session = current.session
        self.current = current
        self.perm_cache = None

    def get_permissions(self):
        """Bellekteki Permission nesnelerini döner.
        Returns:
            object:
        """
        perm_cache = PermissionCache(self.session['role_id'])
        return perm_cache.get() or self.get_role().get_permissions()

    def has_permission(self, perm):
        """Verilen ``perm`` Permission nesnesinin rolde bulunup
        bulunmadığını döner.

        Args:
            perm:

        Returns:
            bool:

        """
        # return True
        return perm in self.get_permissions()

    def get_user(self):
        """Session'da bir kullanıcı varsa sesion'daki kullanıcıyı yoksa
        boş bir kullanıcı nesnesi döner.

        Returns:
            object: User nesnesi

        """
        # if 'user_data' in self.session:
        #     user = User()
        #     user.set_data(self.session['user_data'], from_db=True)
        #     user.key = self.session['user_id']
        # elif 'user_id' in self.session:
        if 'user_id' in self.session:
            user = User.objects.get(self.session['user_id'])
        else:
            user = User()
        return user

    def set_user(self, user):
        """
        Kullanıcı datasını session'a yazar.

        Args:
            user: User nesnesi


        """
        self.session['user_id'] = user.key
        self.session['user_data'] = user.clean_value()
        role = user.last_login_role()
        self.session['role_id'] = role.key
        self.current.role_id = role.key
        self.current.user_id = user.key
        self.perm_cache = PermissionCache(role.key)
        self.session['permissions'] = role.get_permissions()

    def get_role(self):
        """session'da bir role_id varsa bu id'deki Role nesnesini döner.
        Yoksa session objesini siler ve PermissionDenied hatası verir.

        Returns:
            object: Role nesnesi
        Raises:
            PermissionDenied:

        """
        if 'role_id' in self.session:
            return Role.objects.get(self.session['role_id'])
        else:
            # TODO: admins should be informed about a user without a role
            self.session.delete()
            raise PermissionDenied("Your don't have a \"Role\" in this system")

    def authenticate(self, username, password):
        """Kullanıcı adı ve şifresini kontrol eder ve eğer doğruysa
        kullanıcı datasını session'a yazar.

        Args:
            username:
            password:

        Returns:
            bool:

        """
        user = User.objects.get(username=username)
        is_login_ok = user.check_password(password)
        if is_login_ok:
            self.set_user(user)
        else:
            pass
            # TODO: failed login attempts for a user should be counted
            # for prevention of brute force attacks

        return is_login_ok


# invalidate permission cache on crud updates on Role and AbstractRole models
@receiver(crud_post_save)
def clear_perm_cache(sender, *args, **kwargs):
    """Verilen sender nesnesine bağlı Permission nesnelerini cache
    bellekten siler.

    Eğer sender nesnesi bir AbstractRole ise PermissionCache'i temizler.

    Args:
        sender:
        *args:
        **kwargs:

    """
    if sender.model_class.__name__ == 'Role':
        PermissionCache(kwargs['object'].key).delete()
    elif sender.model_class.__name__ == 'AbstractRole':
        PermissionCache.flush()


def ulakbus_permissions():
    """Bu metot Ulakbus'e ait tüm yetkileri birleştirerek döner.

    Returns:
        list: Ulakbus'e ait tüm yetkiler

    """
    default_perms = get_all_permissions()
    from ulakbus.views.reports import ReporterRegistry
    report_perms = ReporterRegistry.get_permissions()
    return default_perms + report_perms



# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

from ulakbus.services.ulakbus_service import UlakbusService
from ulakbus.services.personel.hitap.hitap_helper import HitapHelper
import os
import urllib2
from json import dumps

"""HITAP Guncelle Servisi

Hitap guncelle (insert) servislerinin kalıtılacağı
abstract HITAP Guncelle servisini içeren modül.


Attributes:
    H_USER (str): Hitap kullanıcı adı
    H_PASS (str): Hitap kullanıcı şifresi

"""

H_USER = os.environ["HITAP_USER"]
H_PASS = os.environ["HITAP_PASS"]


class HITAPGuncelle(UlakbusService):
    """
    Hitap Güncelleme servislerinin kalıtılacağı abstract Zato servisi.

    Güncelleme servisleri gerekli girdileri (Hitap dict, Hitap username, Hitap password)
    Hitap'a yollayıp dönecek cevabı elde etmektedir.

    Attributes:
        service_name (str): İlgili Hitap sorgu servisinin adı
        service_dict (dict): Hitap servisine yollanacak datayı hazırlamak için sözlük.
            Servise gönderilecek verinin alanlarına ait sözlüğü
            ve tarih filtresi uygulanacak ve servis tarafında gerekli olan alanların
            listesini içerir.

    """
    HAS_CHANNEL = False

    def __init__(self):
        self.service_name = ''
        self.service_dict = {'fields': {}, 'date_filter': [], 'required_fields': []}
        super(HITAPGuncelle, self).__init__()

    def handle(self):
        """
        Servis çağrıldığında tetiklenen metod.

        Hitap'a gidecek isteği hazırlayacak ve gelen cevabı elde edecek olan
        request_json fonksiyonunu çağırır.

        """

        self.logger.info("zato service started to work.")
        conn = self.outgoing.soap['HITAP'].conn

        self.request_json(conn)

    def request_json(self, conn):
        """
        Connection bilgisi ve gerekli veriler ile Hitap'ın ilgili servisine
        istekte bulunup gelen cevabı uygun şekilde elde eder.

        Hitap'tan gelen verilerin sisteme uygun şekilde elde edilmesi için, gerekli
        Hitap sözlüğünü yaratan ve tarih alanı gibi belli alanlara uygun filteleri
        uygulayacak olan fonksiyonları çağırır.

        Veriler uygun şekilde elde edildikten sonra servis cevabı (payload) oluşturulur.

        Args:
            conn (zato.outgoing.soap.conn): Zato soap outgoing bağlantısı

        Raises:
            AttributeError: İlgili servis veya bean Hitap'ta bulunmayabilir.
            urllib2.URLError: Servis yanıt vermiyor.

        """

        status = "error"
        hitap_response = False
        request_data = {}

        try:
            # connection for hitap
            with conn.client() as client:
                # hitap response
                service_call_name = self.service_mapper(self.service_name)

                if service_call_name:
                    request_data = client.factory.create(service_call_name)

                # filtering for some fields
                if 'date_filter' in self.service_dict:
                    self.date_filter(self.service_dict)
                self.custom_filter(self.service_dict)

                if 'required_fields' in self.service_dict:
                    required_field_check = HitapHelper()
                    required_field_check.check_required_data(self.service_dict)

                for dict_element in self.service_dict['fields']:
                    request_data[dict_element] = self.service_dict[dict_element]

                self.logger.info("%s started to work." % self.service_name)

                hitap_response = getattr(client.service, self.service_name)(request_data,
                                                                            kullaniciAd=H_USER,
                                                                            sifre=H_PASS)
                if hitap_response:
                    hitap_response = self.create_hitap_json(hitap_response)

            status = "ok"

        except AttributeError:
            self.logger.exception("AttributeError")
            status = "error"

        except urllib2.URLError:
            self.logger.exception("Service unavailable!")
            status = "error"

        finally:
            self.response.payload = {'status': status, 'result': hitap_response}

    def create_hitap_json(self, hitap_service):
        """Güncelleme servisinden dönen veriyi JSON formatına döndürür.
        Converts SOAP call result object into JSON

        """
        dict_result = dict((name, getattr(hitap_service, name)) for name in dir(hitap_service)
                           if not name.startswith('__'))

        self.logger.info("hitap_service json created.")

        return dumps(dict_result)

    def date_filter(self, hitap_dict):
        """
        Yerel kayıttaki tarih alanlarını HITAP servisine uygun biçime getirir.

        Hitap'ta tarih alanları için ``0001-01-01`` değeri boş değer anlamına gelirken,
        yerelde ``01.01.1900`` şeklindedir.

        Yerelde GG-AA-YYYY formatında tutulurken, HITAP servisi üzerindeki geçerli format YYYY-AA-GG
        şeklindedir.

        Args:
            hitap_dict (List[dict]): Hitap verisini yerele uygun biçimde tutan sözlük listesi

        """
        if hitap_dict['date_filter']:
            from datetime import datetime
            for record in hitap_dict:
                for field in hitap_dict['date_filter']:
                    if record[field] == "01.01.1900":
                        record[field] = '0001-01-01'
                    else:
                        date_format = datetime.strptime(record[field], "%d.%m.%Y")
                        record[field] = date_format.strftime("%Y-%m-%d")

    def custom_filter(self, hitap_dict):
        """
        Hitap sözlüğüne uygulanacak ek filtreleri (varsa) gerçekleştirir.

        Gerçekleştirimi bu servisten kalıtılan servislerde yapılmaktadır.

        Args:
            hitap_dict (List[dict]): Hitap verisini yerele uygun biçimde tutan sözlük listesi

        """
        pass

    def service_mapper(self, service_name):

        """Hitap'a yapılacak olan Update çağrılarında gerekli datanın oluşmasını sağlamak için
        Servis ve ServisBean'lerini eşleyen method.

        """
        services_dict = {"HizmetAcikSureUpdate": "ns1:HizmetAcikSureServisBean",
                         "HizmetAskerlikUpdate": "ns1:HizmetAskerlikServisBean",
                         "HizmetBirlestirmeUpdate": "ns1:HizmetBirlestirmeServisBean",
                         "HizmetBorclanmaUpdate": "ns1:HizmetBorclanmaServisBean",
                         "HizmetCetvelUpdate": "ns1:HizmetCetveliServisBean",
                         "HizmetIHSUpdate": "ns1:HizmetIHSServisBean",
                         "HizmetKursUpdate": "ns1:HizmetEgitimKursServisBean",
                         "HizmetMahkemeUpdate": "ns1:HizmetMahkemeServisBean",
                         "HizmetNufusUpdate": "ns1:HizmetNufusServisBean",
                         "HizmetOkulUpdate": "ns1:HizmetEgitimOkulServisBean",
                         "HizmetTazminatUpdate": "ns1:HizmetTazminatServisBean",
                         "HizmetUnvanUpdate": "ns1:HizmetUnvanServisBean",
                         "hizmetIstisnaiIlgiUpdate": "ns1:HizmetIstisnaiIlgiServisBean"}

        if service_name in services_dict:
            return services_dict[service_name]
        else:
            return False



# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP Mahkeme Ekle

Hitap'a personelin Mahkeme bilgilerinin eklenmesini yapar.

"""

from ulakbus.services.personel.hitap.hitap_ekle import HITAPEkle


class HizmetMahkemeEkle(HITAPEkle):
    """
    HITAP Ekleme servisinden kalıtılmış Hizmet Mahkeme Bilgi Ekleme servisi

    """
    HAS_CHANNEL = True

    def handle(self):
        """Servis çağrıldığında tetiklenen metod.

        Attributes:
            service_name (str): İlgili Hitap sorgu servisinin adı
            service_dict (dict): Request yoluyla gelen kayıtlar,
                    HizmetMahkemeInsert servisinin alanlarıyla eşlenmektedir.
                    Filtreden geçecek tarih alanları ve servis tarafında gerekli olan
                    alanlar listede tutulmaktadır.

        """

        self.service_name = 'HizmetMahkemeInsert'
        self.service_dict = {
            'fields': {
                'tckn': self.request.payload.get('tckn', ''),
                'mahkemeAd': self.request.payload.get('mahkeme_ad', ''),
                'sebep': self.request.payload.get('sebep', ''),
                'kararTarihi': self.request.payload.get('karar_tarihi', ''),
                'kararSayisi': self.request.payload.get('karar_sayisi', ''),
                'kesinlesmeTarihi': self.request.payload.get('kesinlesme_tarihi', ''),
                'asilDogumTarihi': self.request.payload.get('asil_dogum_tarihi', ''),
                'tashihDogumTarihi': self.request.payload.get('tashih_dogum_tarihi', ''),
                'gecerliDogumTarihi': self.request.payload.get('gecerli_dogum_tarihi', ''),
                'asilAd': self.request.payload.get('asil_ad', ''),
                'tashihAd': self.request.payload.get('tashih_ad', ''),
                'asilSoyad': self.request.payload.get('asil_soyad', ''),
                'tashihSoyad': self.request.payload.get('tashih_soyad', ''),
                'aciklama': self.request.payload.get('aciklama', ''),
                'gunSayisi': self.request.payload.get('gun_sayisi', ''),
                'kurumOnayTarihi': self.request.payload.get('kurum_onay_tarihi', '')
            },
            'date_filter': ['kesinlesmeTarihi', 'asilDogumTarihi', 'tashihDogumTarihi',
                            'gecerliDogumTarihi', 'kurumOnayTarihi'],
            'required_fields': ['tckn', 'mahkemeAd', 'sebep', 'kararTarihi', 'kararSayisi',
                                'kurumOnayTarihi']
        }
        super(HizmetMahkemeEkle, self).handle()



# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
from pyoko.exceptions import ObjectDoesNotExist
from ulakbus.lib.view_helpers import convert_model_object_titlemap_item
from ulakbus.models import Donem, OgretimYili
from zengine.forms import JsonForm
from zengine.forms import fields
from zengine.views.crud import CrudView
from zengine.lib.translation import gettext as _, gettext_lazy


def donemler():
    """
    Dönem seçim formunu dinamik olarak dolduran method.
    Güncel dönemden önceki ve sonraki 2 şer adet dönemle birlikte bir seçim
    listesi oluşturur.
    """
    ds = []  # return list
    td = []  # tum donemler, onceki guncel ve sonrakilerden liste
    gd = Donem.guncel_donem()
    td.extend(gd.onceki_donemler(2))
    td.append(gd)
    td.extend(gd.sonraki_donemler(2))
    for donem in td:
        ds.append(convert_model_object_titlemap_item(donem))
    return ds


class DonemSecForm(JsonForm):
    guncel_donem = fields.Integer(gettext_lazy(u"Dönem Seçiniz"), choices=donemler)
    kaydet = fields.Button(gettext_lazy(u'Kaydet'))


class GuncelDonemDegistirme(CrudView):
    """
    Güncel Dönem Değiştirme iş akışı 3 adımdan oluşur.

    Güncel Dönem Seç:
    SelectBox içinde gelen dönemler bir güncel dönem olarak seçilir.

    Güncel Dönem Kaydet:
    Seçilen güncel dönem kaydedilir.

    Bilgi Ver:
    İşlemin başarılı olduğuna dair bilgilendirme mesajı verilir.

    """

    class Meta:
        model = 'Donem'

    def guncel_donem_sec(self):
        """
        SelectBox içinde gelen dönemler bir güncel dönem olarak seçilir.
        SelectBox içindeki dönemler, güncel dönemden önceki 2 dönem, güncel
        dönemden sonraki iki dönem  ve güncel dönem sıralanır.

        """

        _form = DonemSecForm(title=_(u'Güncel Dönem Seçiniz'), current=self.current)

        # formun güncel dönemi seçili hale getirilir.
        _form.guncel_donem = Donem.guncel_donem().key

        self.form_out(_form)

    def guncel_donem_kaydet(self):
        """
        Seçilen güncel dönem kaydedilir.

        """

        self.current.task_data['donem_key'] = self.current.input['form']['guncel_donem']
        donem = Donem.objects.get(self.current.task_data['donem_key'])
        try:
            ogretim_yili = OgretimYili.objects.get(yil=int(donem.baslangic_tarihi.year))
        except ObjectDoesNotExist:
            ogretim_yili = OgretimYili(yil=int(donem.baslangic_tarihi.year))
            ogretim_yili.save()

        donem.ogretim_yili = ogretim_yili
        donem.guncel = True
        meta = {'user_id': self.current.user_id,
                'role_id': self.current.role_id,
                'wf_name': self.current.workflow_name,
                'task_name': self.current.task_name}
        index_fields = [('user_id', 'bin'), ('wf_name', 'bin')]
        donem.save(meta=meta, index_fields=index_fields)

    def bilgi_ver(self):
        """
        İşlemin başarılı olduğuna dair bilgilendirme mesajı verilir.

        """

        donem = Donem.objects.get(self.current.task_data['donem_key'])
        self.current.output['msgbox'] = {
            'type': 'info', "title": _(u'Güncel Dönem Değiştirme'),
            "msg": _(u'{donem} dönemi güncel dönem olarak atanmıştır').format(donem=donem)}



# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP Zato Service Wrapper

Includes Zato Service Wrapper and wrapper functions for services
such as Hitap, Mernis, KPS etc.


Example:

    .. code-block:: python

        from zato_wrapper_class import HitapHizmetCetveliGetir
        zs = HitapHizmetCetvelGetir(tckn="12345678900")
        response = zs.zato_request()  # list, with lines of hizmet cetveli


    .. code-block:: python

        from zato_wrapper_class import HitapHizmetCetveliSenkronizeEt
        zs = HitapHizmetCetveliSenkronizeEt(tckn="12345678900")
        response = zs.zato_request()  # 'ok' or 'error'

"""

from ulakbus import settings
import requests
import json
from .zato_url_paths import service_url_paths


class ZatoService(object):
    """
    Simple zato service wrapper class.
    You can write your zato services extending this class.

    This class simply needs some parameter to be set
    as you see in init payload and service uri.

    Ping service returns just 'ok' which can be used as an health check
    to determine zato servers is alive or not.

    ``service_uri`` and ``payload`` parameters especially is
    set by extending class' __init__.

    Attributes:
        payload (str): an empty dict as default
        service_uri (str): "ping" as default

    """

    def __init__(self):
        self.payload = "{}"
        self.service_uri = "ping"

    def get_uri(self):
        """
        Simply returns full uri of zato service object.
        It uses ``ZATO_SERVER`` from settings module.

        The ``ZATO_SERVER`` can be set as environment variable as below::

            export ZATO_SERVER='http://127.0.0.1/'

        Returns:
            (str): unique identification string of service on zato services
            including join of zato server url (generally a load balancer url)
            and service name on zato.

        """

        return '/'.join([settings.ZATO_SERVER, self.service_uri])

    def zato_request(self):
        """
        Makes zato requests. Zato expects payloads as json over POST method
        and sends back json. Return json contains two parts
        one is status and the other is result.

        Status part is can be ``ok`` or ``error`` depends on
        what happens in zato servers while running. Result part is
        the data part which is expected by consumer.

        Returns:
            str: if requests fails, returns ``None``
            or simply string of zato service response payload

        """

        uri = self.get_uri()
        payload = json.loads(self.payload)
        r = requests.post(uri, data=json.dumps(payload))
        if r.status_code == 200:
            response = r.json()
            r.close()
            try:
                if response['status'] == 'ok':
                    return self.rebuild_response(response['result'])
                else:
                    # all zato internal errors will be handled here,
                    # riak error, connection error etc..
                    raise Exception("your service request failed with error %s"
                                    % response['result'])
            except KeyError:
                raise Exception("your service response contains no status code, "
                                "check your zato service package")

        if r.status_code == 404:
            raise Exception("Service called '%s' is not defined on zato "
                            "servers or service_uri changed" % self.service_uri)
        # other than 404 errors will be handled here,
        # such as unauthorized requests, permission denied or etc..
        else:
            raise Exception("Status code is something different 200 or 404 which is %s, "
                            "this means something really went bad, check zato server logs"
                            % r.status_code)

    def rebuild_response(self, response_data):
        """
        Rebuild response data

        Args:
            response_data (dict): contains data returned from service

        Returns:
            response_data (dict): reformatted data compatible with our data models

        """
        return response_data


class TcknService(ZatoService):
    @staticmethod
    def check_turkish_identity_number(tckn):
        """
        Türkiye Cumhuriyeti Kimlik Numarası geçerlilik kontrolü.
        11 karakter uzunluğunda karakter dizisi olmalı.

        Args:
            tckn (str): 11 karakter uzunluğunda TCKN.

        Returns:
            str: TCKN

        """

        if not tckn:
            raise Exception("tckn can not be empty")

        if type(tckn) is not str:
            raise TypeError("tckn must be string which is %s" % type(tckn))

        if len(tckn) != 11:
            raise Exception("tckn length must be 11")

        return tckn


class HitapService(TcknService):
    pass


class HitapServiceError(Exception):
    pass


class HitapAcikSureGetir(HitapService):
    """
    Hitap üzerinden, personelin açık süre hizmet bilgilerini sorgular.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(HitapAcikSureGetir, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)


class HitapAcikSureSenkronizeEt(HitapService):
    """
    Personelin Hitap'taki açık süre hizmet bilgilerinin,
    yereldeki kayıtlarla senkronizasyonunu yapar.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(HitapAcikSureSenkronizeEt, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)


class HitapAcikSureEkle(HitapService):
    """
    Personelin açık süre hizmet bilgilerinin Hitap'a,
    eklemesini yapar.

    Args:
      service_payload (dict): Servise gönderilecek olan veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapAcikSureEkle, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapAcikSureGuncelle(HitapService):
    """
    Personelin açık süre hizmet bilgilerinin Hitap üzerinde
    guncellemesini yapar.

    Args:
      service_payload (dict): Servise gönderilecek olan veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapAcikSureGuncelle, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapAcikSureSil(HitapService):
    """
    Personelin açık süre hizmet bilgilerinin Hitap üzerinden
    silme işlemini yapar.

    Args:
      service_payload (dict): Servise gönderilecek olan veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapAcikSureSil, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapAskerlikGetir(HitapService):
    """
    Hitap üzerinden, personelin askerlik bilgilerini sorgular.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(HitapAskerlikGetir, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)


class HitapAskerlikSenkronizeEt(HitapService):
    """
    Personelin Hitap'taki askerlik bilgilerinin,
    yereldeki kayıtlarla senkronizasyonunu yapar.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(HitapAskerlikSenkronizeEt, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)


class HitapAskerlikEkle(HitapService):
    """
    Personelin askerlik bilgilerinin Hitap'a eklemesini yapar.

    Args:
      service_payload (dict): Servise gönderilecek olan veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapAskerlikEkle, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapAskerlikGuncelle(HitapService):
    """
    Personelin askerlik bilgilerinin Hitap üzerinde
    guncellemesini yapar.

    Args:
      service_payload (dict): Servise gönderilecek olan veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapAskerlikGuncelle, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapAskerlikSil(HitapService):
    """
    Personelin askerlik bilgilerinin Hitap üzerinden
    silme işlemini yapar.

    Args:
      service_payload (dict): Servise gönderilecek olan veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapAskerlikSil, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapBirlestirmeGetir(HitapService):
    """
    Hitap üzerinden, personelin hizmet birleştirme bilgilerini sorgular.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(HitapBirlestirmeGetir, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)


class HitapBirlestirmeSenkronizeEt(HitapService):
    """
    Personelin Hitap'taki hizmet birleştirme bilgilerinin,
    yereldeki kayıtlarla senkronizasyonunu yapar.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(HitapBirlestirmeSenkronizeEt, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)


class HitapBirlestirmeEkle(HitapService):
    """
    Personelin hizmet birleştirme bilgilerinin Hitap'a
    eklemesini yapar.

    Args:
      service_payload (dict): Servise gönderilecek olan veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapBirlestirmeEkle, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapBirlestirmeGuncelle(HitapService):
    """
    Personelin hizmet birleştirme bilgilerinin Hitap üzerinde
    güncellemesini yapar.

    Args:
      service_payload (dict): Servise gönderilecek olan veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapBirlestirmeGuncelle, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapBirlestirmeSil(HitapService):
    """
    Personelin hizmet birleştirme bilgilerinin Hitap üzerinde
    silme işlemini yapar.

    Args:
      service_payload (dict): Servise gönderilecek olan veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapBirlestirmeSil, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapBorclanmaGetir(HitapService):
    """
    Hitap üzerinden, personelin borçlanma bilgilerini sorgular.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(HitapBorclanmaGetir, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)


class HitapBorclanmaSenkronizeEt(HitapService):
    """
    Personelin Hitap'taki hizmet borçlanma bilgilerinin,
    yereldeki kayıtlarla senkronizasyonunu yapar.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(HitapBorclanmaSenkronizeEt, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)


class HitapBorclanmaEkle(HitapService):
    """
    Personelin hizmet borçlanma bilgilerinin Hitap'a
    eklemesini yapar.

    Args:
      service_payload (dict): Servise gönderilecek olan veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapBorclanmaEkle, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapBorclanmaGuncelle(HitapService):
    """
    Personelin hizmet borçlanma bilgilerinin Hitap üzerinde
    güncellemesini yapar.

    Args:
      service_payload (dict): Servise gönderilecek olan veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapBorclanmaGuncelle, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapBorclanmaSil(HitapService):
    """
    Personelin hizmet borçlanma bilgilerinin Hitap üzerinden
    silme işlemini yapar.

    Args:
      service_payload (dict): Servise gönderilecek olan veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapBorclanmaSil, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapHizmetCetveliGetir(HitapService):
    """
    Hitap üzerinden, personelin hizmet kaydı bilgilerini sorgular.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(HitapHizmetCetveliGetir, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)


class HitapHizmetCetveliSenkronizeEt(HitapService):
    """
    Personelin Hitap'taki hizmet kaydı bilgilerinin,
    yereldeki kayıtlarla senkronizasyonunu yapar.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(HitapHizmetCetveliSenkronizeEt, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)


class HitapHizmetCetveliEkle(HitapService):
    """
    Personelin hizmet kaydı bilgilerinin Hitap'a
    eklemesini yapar.

    Args:
      service_payload (dict): Servise gönderilecek olan veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapHizmetCetveliEkle, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapHizmetCetveliGuncelle(HitapService):
    """
    Personelin hizmet kaydı bilgilerinin Hitap üzerinde
    güncellemesini yapar.

    Args:
      service_payload (dict): Servise gönderilecek olan veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapHizmetCetveliGuncelle, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapHizmetCetveliSil(HitapService):
    """
    Personelin hizmet kaydı bilgilerinin Hitap üzerinde
    silme işlemini yapar.

    Args:
      service_payload (dict): Servise gönderilecek olan veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapHizmetCetveliSil, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapIHSGetir(HitapService):
    """
    Hitap üzerinden, personelin itibari hizmet süresi zammı
    bilgilerini sorgular.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(HitapIHSGetir, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)


class HitapIHSSenkronizeEt(HitapService):
    """
    Personelin Hitap'taki itibari hizmet süresi zammı bilgilerinin,
    yereldeki kayıtlarla senkronizasyonunu yapar.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(HitapIHSSenkronizeEt, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)


class HitapIHSEkle(HitapService):
    """Hitap'a personelin itibari hizmet süresi zammı
    bilgisi ekler.

    Args:
      service_payload (dict): Servise gönderilecek veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapIHSEkle, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapIHSGuncelle(HitapService):
    """Hitap üzerinde personelin itibari hizmet süresi zammı
    bilgilerini günceller.

    Args:
      service_payload (dict): Servise gönderilecek veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapIHSGuncelle, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapIHSSil(HitapService):
    """Hitap üzerinde personelin itibari hizmet süresi zammı
    bilgilerini siler.

    Args:
      service_payload (dict): Servise gönderilecek veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapIHSSil, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapIstisnaiIlgiGetir(HitapService):
    """
    Hitap üzerinden personelin istisnai ilgi
    bilgilerini sorgular.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(HitapIstisnaiIlgiGetir, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)


class HitapIstisnaiIlgiSenkronizeEt(HitapService):
    """
    Personelin Hitap'taki istisnai ilgi bilgilerinin,
    yereldeki kayıtlarla senkronizasyonunu yapar.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(HitapIstisnaiIlgiSenkronizeEt, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)


class HitapIstisnaiIlgiEkle(HitapService):
    """Hitap'a personelin istisnai ilgi bilgilerini ekler.

    Args:
      service_payload (dict): Servise gönderilecek veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapIstisnaiIlgiEkle, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapIstisnaiIlgiGuncelle(HitapService):
    """Hitap üzerinde personelin istisnai ilgi bilgilerini
     günceller.

    Args:
      service_payload (dict): Servise gönderilecek veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapIstisnaiIlgiGuncelle, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapIstisnaiIlgiSil(HitapService):
    """Hitap üzerinde personelin istisnai ilgi bilgilerini siler.

    Args:
      service_payload (dict): Servise gönderilecek veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapIstisnaiIlgiSil, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapKursGetir(HitapService):
    """
    Hitap üzerinden, personelin kurs bilgilerini sorgular.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(HitapKursGetir, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)


class HitapKursSenkronizeEt(HitapService):
    """
    Personelin Hitap'taki kurs bilgilerinin,
    yereldeki kayıtlarla senkronizasyonunu yapar.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(HitapKursSenkronizeEt, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)


class HitapKursEkle(HitapService):
    """Hitap'a personelin kurs bilgilerini ekler.

    Args:
      service_payload (dict): Servise gönderilecek veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapKursEkle, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapKursGuncelle(HitapService):
    """Personelin Hitap'taki kurs bilgilerini günceller.

    Args:
      service_payload (dict): Servise gönderilecek veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapKursGuncelle, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapKursSil(HitapService):
    """Personelin Hitap'taki kurs bilgilerini siler.

    Args:
      service_payload (dict): Servise gönderilecek veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapKursSil, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapMahkemeGetir(HitapService):
    """
    Hitap üzerinden, personelin mahkeme bilgilerini sorgular.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(HitapMahkemeGetir, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)


class HitapMahkemeSenkronizeEt(HitapService):
    """
    Personelin Hitap'taki mahkeme bilgilerinin,
    yereldeki kayıtlarla senkronizasyonunu yapar.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(HitapMahkemeSenkronizeEt, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)


class HitapMahkemeGuncelle(HitapService):
    """Personelin Hitap'taki mahkeme bilgilerini gunceller.

    Args:
      service_payload (dict): Servise gönderilecek veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapMahkemeGuncelle, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapMahkemeEkle(HitapService):
    """Personelin mahkeme bilgilerini Hitap'a ekler.

    Args:
      service_payload (dict): Servise gönderilecek veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapMahkemeEkle, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapMahkemeSil(HitapService):
    """Personelin Hitap'taki mahkeme bilgilerini siler.

    Args:
      service_payload (dict): Servise gönderilecek veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapMahkemeSil, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapNufusGetir(HitapService):
    """
    Hitap üzerinden, personelin nüfus bilgilerini sorgular.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(HitapNufusGetir, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)


class HitapNufusSenkronizeEt(HitapService):
    """
    Personelin Hitap'taki nüfus bilgilerinin,
    yereldeki kayıtlarla senkronizasyonunu yapar.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(HitapNufusSenkronizeEt, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)


class HitapNufusEkle(HitapService):
    """Personelin nufus bilgilerini Hitap'a ekler.

    Args:
      service_payload (dict): Servise gönderilecek veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapNufusEkle, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapNufusGuncelle(HitapService):
    """Personelin  Hitap'ta bulunan nufus bilgilerini gunceller.

    Args:
      service_payload (dict): Servise gönderilecek veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapNufusGuncelle, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapNufusSil(HitapService):
    pass


class HitapOkulGetir(HitapService):
    """
    Hitap üzerinden, personelin okul bilgilerini sorgular.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(HitapOkulGetir, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)


class HitapOkulSenkronizeEt(HitapService):
    """
    Personelin Hitap'taki okul bilgilerinin,
    yereldeki kayıtlarla senkronizasyonunu yapar.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(HitapOkulSenkronizeEt, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)


class HitapOkulEkle(HitapService):
    """Personelin okul bilgilerini Hitap'a ekler.

    Args:
      service_payload (dict): Servise gönderilecek veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapOkulEkle, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapOkulGuncelle(HitapService):
    """Personelin Hitap'ta bulunan okul bilgilerini gunceller.

    Args:
      service_payload (dict): Servise gönderilecek veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapOkulGuncelle, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapOkulSil(HitapService):
    """Personelin Hitap'ta bulunan okul bilgilerini siler.

    Args:
      service_payload (dict): Servise gönderilecek veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapOkulSil, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapTazminatGetir(HitapService):
    """
    Hitap üzerinden, personelin tazminat bilgilerini sorgular.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(HitapTazminatGetir, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)


class HitapTazminatSenkronizeEt(HitapService):
    """
    Personelin Hitap'taki tazminat bilgilerinin,
    yereldeki kayıtlarla senkronizasyonunu yapar.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(HitapTazminatSenkronizeEt, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)


class HitapTazminatEkle(HitapService):
    """Personelin tazminat bilgilerini Hitap'a ekler.

    Args:
      service_payload (dict): Servise gönderilecek veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapTazminatEkle, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapTazminatGuncelle(HitapService):
    """Personelin Hitap'ta bulunan tazminat bilgilerini günceller.

    Args:
      service_payload (dict): Servise gönderilecek veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapTazminatGuncelle, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapTazminatSil(HitapService):
    """Personelin Hitap'ta bulunan tazminat bilgilerini siler.

    Args:
      service_payload (dict): Servise gönderilecek veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapTazminatSil, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapUnvanGetir(HitapService):
    """
    Hitap üzerinden, personelin ünvan bilgilerini sorgular.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(HitapUnvanGetir, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)


class HitapUnvanSenkronizeEt(HitapService):
    """
    Personelin Hitap'taki ünvan bilgilerinin,
    yereldeki kayıtlarla senkronizasyonunu yapar.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(HitapUnvanSenkronizeEt, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)


class HitapUnvanEkle(HitapService):
    """Personelin ünvan bilgilerini Hitap'a ekler.

    Args:
      service_payload (dict): Servise gönderilecek veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapUnvanEkle, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapUnvanGuncelle(HitapService):
    """Personelin Hitap'ta bulunan ünvan bilgilerini günceller.

    Args:
      service_payload (dict): Servise gönderilecek veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapUnvanGuncelle, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class HitapUnvanSil(HitapService):
    """Personelin Hitap'ta bulunan ünvan bilgilerini siler.

    Args:
      service_payload (dict): Servise gönderilecek veri

    Attributes:
        service_uri (str): İlgili Hitap servisinin adı
        payload (str): Servis verisi

    """

    def __init__(self, service_payload={}):
        super(HitapUnvanSil, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dump(service_payload)


class MernisKimlikBilgileriGetir(TcknService):
    """
    Personelin Mernis Kimlik bilgilerini sorgular.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili servisin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(MernisKimlikBilgileriGetir, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)

    def rebuild_response(self, response_data):
        """
        Kimlik Bilgileri servisinden dönen datayı modellerimize uygun hale getirir.

        Args:
            response_data (dict): contains data returned from service

        Returns:
            response_data (dict): reformatted data compatible with our data models

        """
        ret = {}

        try:
            kb = response_data['KisiBilgisi']

            ret['tckn'] = kb['TCKimlikNo']
            ret['ad'] = kb['TemelBilgisi']['Ad']
            ret['soyad'] = kb['TemelBilgisi']['Soyad']
            ret['cinsiyet'] = kb['TemelBilgisi']['Cinsiyet']['Kod']
            ret['dogum_tarihi'] = '%s.%s.%s' % (
                kb['TemelBilgisi']['DogumTarih']['Gun'], kb['TemelBilgisi']['DogumTarih']['Ay'],
                kb['TemelBilgisi']['DogumTarih']['Yil'])
            ret['dogum_yeri'] = kb['TemelBilgisi']['DogumYer']
            ret['baba_adi'] = kb['TemelBilgisi']['BabaAd']
            ret['ana_adi'] = kb['TemelBilgisi']['AnneAd']
            ret['medeni_hali'] = kb['DurumBilgisi']['MedeniHal']['Kod']
            ret['kayitli_oldugu_il'] = kb['KayitYeriBilgisi']['Il']['Aciklama']
            ret['kayitli_oldugu_ilce'] = kb['KayitYeriBilgisi']['Ilce']['Aciklama']
            ret['kayitli_oldugu_mahalle_koy'] = kb['KayitYeriBilgisi']['Cilt']['Aciklama']
            ret['kayitli_oldugu_cilt_no'] = kb['KayitYeriBilgisi']['Cilt']['Kod']
            ret['kayitli_oldugu_aile_sira_no'] = kb['KayitYeriBilgisi']['AileSiraNo']
            ret['kayitli_oldugu_sira_no'] = kb['KayitYeriBilgisi']['BireySiraNo']
        except KeyError:
            ret['hata'] = True

        return ret


class MernisCuzdanBilgileriGetir(TcknService):
    """
    Personelin Mernis üzerinden nüfus cüzdanı bilgilerini sorgular.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili servisin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(MernisCuzdanBilgileriGetir, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)

    def rebuild_response(self, response_data):
        """
        Mernis Cüzdan Bilgileri servisinden dönen datayı modellerimize uygun hale getirir.

        Args:
            response_data (dict): contains data returned from service

        Returns:
            response_data (dict): reformatted data compatible with our data models

        """
        ret = {}

        try:
            kb = response_data['CuzdanBilgisi']
            ret['tckn'] = kb['TCKimlikNo']
            ret['cuzdan_seri'] = kb['SeriNo'][0:3]
            ret['cuzdan_seri_no'] = kb['SeriNo'][3:]
            ret['kimlik_cuzdani_verildigi_yer'] = kb['VerildigiIlce']['b:Aciklama']
            ret['kimlik_cuzdani_verilis_nedeni'] = kb['CuzdanVerilmeNeden']['b:Aciklama']
            ret['kimlik_cuzdani_kayit_no'] = kb['KayitNo']
            ret['kimlik_cuzdani_verilis_tarihi'] = '%s.%s.%s' % (
                kb['VerilmeTarih']['b:Gun'], kb['VerilmeTarih']['b:Ay'],
                kb['VerilmeTarih']['b:Yil'])
        except KeyError:
            ret['hata'] = True

        return ret


class KPSAdresBilgileriGetir(TcknService):
    """
    Personelin KPS Adres bilgilerini sorgular.

    Args:
        tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

    Attributes:
        service_uri (str): İlgili servisin adı
        payload (str): Servis verisi

    """

    def __init__(self, tckn=""):
        super(KPSAdresBilgileriGetir, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = '{"tckn":"%s"}' % self.check_turkish_identity_number(tckn)

    def rebuild_response(self, response_data):
        """
        KPS Adres Bilgileri servisinden dönen datayı modellerimize uygun hale getirir.

        Args:
            response_data (dict): contains data returned from service

        Returns:
            response_data (dict): reformatted data compatible with our data models

        """
        ret = {}
        try:
            kb = response_data['KimlikNoileKisiAdresBilgileri']['YerlesimYeriAdresi']
            ret['ikamet_adresi'] = kb['AcikAdres']
            ret['ikamet_il'] = kb['IlIlceMerkezAdresi']['Il']
            ret['ikamet_ilce'] = kb['IlIlceMerkezAdresi']['Ilce']
        except KeyError:
            ret['hata'] = True

        return ret

class DersProgramiOlustur(ZatoService):
    """
    dp = DersProgramiOlustur(service_payload={"bolum": 123445})
    response = dp.zato_request()
    """

    def __init__(self, service_payload={}):
        super(ZatoService, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dumps(service_payload)


class SinavProgramiOlustur(ZatoService):
    """
    dp = SinavProgramiOlustur(service_payload={"bolum": 123445})
    response = dp.zato_request()
    """

    def __init__(self, service_payload={}):
        super(ZatoService, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dumps(service_payload)

class E_PostaYolla(ZatoService):
    def __init__(self, service_payload={}):
        super(ZatoService, self).__init__()
        self.service_uri = service_url_paths[self.__class__.__name__]["url"]
        self.payload = json.dumps(service_payload)



# -*- coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

from ulakbus.services.ulakbus_service import UlakbusService
import json
import uuid
import httplib
import os

"""
UYARI : Lütfen bu kod içerisindeki uzun satırları formatlamayın. Gönderilen xml içersindeki
satırların bozulması durumunda kps tarafında HTTP_STATUS 400 dönmektedir.

"""

__author__ = 'Ali Riza Keles'

DEBUG = os.environ.get('DEBUG', False)
if DEBUG:
    import logging

    httplib.HTTPConnection.debuglevel = 1
    logging.basicConfig()
    logging.getLogger().setLevel(logging.DEBUG)


class NVIService(UlakbusService):

    HAS_CHANNEL = False

    def __init__(self):
        self.service_action = ""
        self.service_xml_body = ""
        self.sso_data = {}
        super(NVIService, self).__init__()

    def handle(self):
        keys = ['nvi_sso_encrypted_data', 'nvi_sso_key_identifier_path',
                'nvi_sso_digest_value', 'nvi_sso_signature',
                'nvi_sso_created', 'nvi_sso_expire']
        for k in keys:
            val = self.kvdb.conn.get(k)
            self.logger.info("k: %s, v: %s, type: %s" % (k, val, type(val)))
            self.sso_data.update({k: self.kvdb.conn.get(k)})
        if not all(self.sso_data.values()):
            self.invoke_sso_service()

        response = self.requestx()
        response_xml = response.read()
        self.response.payload = {"status": "ok" if response.status==200 else response.status, "result": self.xml_to_json(response_xml)}

    def invoke_sso_service(self):
        response = self.invoke('nvi-sts.sts-get-token')
        if response['status'] == 'ok':
            result = json.loads(response['result'])
            self.sso_data.update(result)
            # self.logger.info("DDDDDDDDDDDDD: %s" % result)
            # if response['status'] == 'ok':
            #     for k, v in result.items():
            #         self.sso_data.update({k, v})

    def request_xml(self):
        request_xml = """
        <?xml version="1.0"?><s:Envelope xmlns:s="http://www.w3.org/2003/05/soap-envelope" xmlns:a="http://www.w3.org/2005/08/addressing" xmlns:u="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd">
          <s:Header>
            <a:Action s:mustUnderstand="1">http://kps.nvi.gov.tr%s</a:Action>
            <a:MessageID>urn:uuid:%s</a:MessageID>
            <a:ReplyTo>
              <a:Address>http://www.w3.org/2005/08/addressing/anonymous</a:Address>
            </a:ReplyTo>
            <a:To s:mustUnderstand="1">https://kpsv2.nvi.gov.tr/Services/RoutingService.svc</a:To>
            <o:Security s:mustUnderstand="1" xmlns:o="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd">
              <u:Timestamp u:Id="_0">
                <u:Created>%s</u:Created>
                <u:Expires>%s</u:Expires>
              </u:Timestamp>%s<Signature xmlns="http://www.w3.org/2000/09/xmldsig#">
                <SignedInfo>
                  <CanonicalizationMethod Algorithm="http://www.w3.org/2001/10/xml-exc-c14n#"/>
                  <SignatureMethod Algorithm="http://www.w3.org/2000/09/xmldsig#hmac-sha1"/>
                  <Reference URI="#_0">
                    <Transforms>
                      <Transform Algorithm="http://www.w3.org/2001/10/xml-exc-c14n#"/>
                    </Transforms>
                    <DigestMethod Algorithm="http://www.w3.org/2000/09/xmldsig#sha1"/>
                    <DigestValue>%s</DigestValue>
                  </Reference>
                </SignedInfo>
                <SignatureValue>%s</SignatureValue>
                <KeyInfo>
                  <o:SecurityTokenReference k:TokenType="http://docs.oasis-open.org/wss/oasis-wss-saml-token-profile-1.1#SAMLV1.1" xmlns:k="http://docs.oasis-open.org/wss/oasis-wss-wssecurity-secext-1.1.xsd">
                    <o:KeyIdentifier ValueType="http://docs.oasis-open.org/wss/oasis-wss-saml-token-profile-1.0#SAMLAssertionID">%s</o:KeyIdentifier>
                  </o:SecurityTokenReference>
                </KeyInfo>
              </Signature>
            </o:Security>
          </s:Header>%s</s:Envelope>""" % (self.service_action, str(uuid.uuid1()), self.sso_data['nvi_sso_created'],
                                           self.sso_data['nvi_sso_expire'],
                                           self.sso_data['nvi_sso_encrypted_data'],
                                           self.sso_data['nvi_sso_digest_value'],
                                           self.sso_data['nvi_sso_signature'],
                                           self.sso_data['nvi_sso_key_identifier_path'],
                                           self.service_xml_body)
        return request_xml

    def requestx(self):
        request_xml = self.request_xml().replace('  ', '').replace('\n', '')
        self.logger.info("\n\n rxml: %s \n\n\n" % request_xml)
        conn = httplib.HTTPConnection("services.konya.edu.tr", 3128)
        headers = {"Content-Type": "application/soap+xml; charset=utf-8"}
        conn.request("POST", "https://kpsv2.nvi.gov.tr/Services/RoutingService.svc",
                     request_xml, headers)
        return conn.getresponse()

    def xml_to_json(self, response_xml=None, response_element=None):
        from xml.dom import minidom
        response_dict = {}

        if response_xml:
            root = minidom.parseString(response_xml).getElementsByTagName('SorguSonucu')[0]
        else:
            root = response_element

        for e in root.childNodes:
            if e.hasChildNodes():
                response_dict[e.nodeName] = self.xml_to_json(response_element=e)
            elif e.nodeType == e.TEXT_NODE:
                return e.data

        return response_dict


class KisiSorgulaTCKimlikNo(NVIService):
    """
    NVI Kimlik Bilgileri Servisi
    """

    HAS_CHANNEL = True

    def handle(self):
        tckn = self.request.payload['tckn']
        self.service_action = "/2011/01/01/KisiSorgulaTCKimlikNoServis/ListeleCoklu"
        self.service_xml_body = """
            <env:Body xmlns:env="http://www.w3.org/2003/05/soap-envelope" xmlns:ns1="http://kps.nvi.gov.tr/2011/01/01">
                  <ns1:ListeleCoklu>
                      <ns1:kriterListesi>
                          <ns1:KisiSorgulaTCKimlikNoSorguKriteri>
                              <ns1:TCKimlikNo type="xsd:long">%s</ns1:TCKimlikNo>
                          </ns1:KisiSorgulaTCKimlikNoSorguKriteri>
                      </ns1:kriterListesi>
                  </ns1:ListeleCoklu>
            </env:Body>""" % tckn
        super(KisiSorgulaTCKimlikNo, self).handle()


class CuzdanSorgulaTCKimlikNo(NVIService):
    """
    NVI Kimlik Bilgileri Servisi
    """

    HAS_CHANNEL = True

    def handle(self):
        tckn = self.request.payload['tckn']
        self.service_action = "/2014/09/01/CuzdanSorgulaTCKimlikNoServis/ListeleCoklu"
        self.service_xml_body = """
            <env:Body xmlns:env="http://www.w3.org/2003/05/soap-envelope" xmlns:ns1="http://kps.nvi.gov.tr/2014/09/01">
                <ns1:ListeleCoklu>
                    <ns1:kriterListesi>
                        <ns1:CuzdanSorgulaTCKimlikNoSorguKriteri>
                            <ns1:CuzdanTur>Cuzdan</ns1:CuzdanTur>
                            <ns1:TCKimlikNo>%s</ns1:TCKimlikNo>
                        </ns1:CuzdanSorgulaTCKimlikNoSorguKriteri>
                    </ns1:kriterListesi>
                </ns1:ListeleCoklu>
            </env:Body>""" % tckn
        super(CuzdanSorgulaTCKimlikNo, self).handle()


class YabanciKisiSorgula(NVIService):
    """
    NVI Kimlik Bilgileri Servisi
    """

    HAS_CHANNEL = True

    def handle(self):
        tckn = self.request.payload['tckn']
        self.service_action = "/2013/06/01/YbKisiSorgulaYbKimlikNoServis/ListeleCoklu"
        self.service_xml_body = """
              <env:Body xmlns:env="http://www.w3.org/2003/05/soap-envelope" xmlns:ns2="http://kps.nvi.gov.tr/2013/06/01" xmlns:ns1="http://kps.nvi.gov.tr/2011/01/01">
                  <ns2:ListeleCoklu>
                      <ns2:kriterListesi>
                          <ns1:YbKisiSorgulaYbKimlikNoSorguKriteri>
                              <ns1:KimlikNo>%s</ns1:KimlikNo>
                          </ns1:YbKisiSorgulaYbKimlikNoSorguKriteri>
                      </ns2:kriterListesi>
                  </ns2:ListeleCoklu>
              </env:Body>""" % tckn
        super(YabanciKisiSorgula, self).handle()


class AdresSorgula(NVIService):
    """
    Adres Sorgulama
    """

    HAS_CHANNEL = True

    def handle(self):
        tckn = self.request.payload['tckn']
        self.service_action = "/2015/07/01/KimlikNoSorgulaAdresServis/Sorgula"
        self.service_xml_body = """
            <env:Body xmlns:env="http://www.w3.org/2003/05/soap-envelope" xmlns:ns2="http://kps.nvi.gov.tr/2015/07/01" xmlns:ns1="http://kps.nvi.gov.tr/2011/01/01">
                    <ns2:Sorgula>
                        <ns2:kriterListesi>
                            <ns1:KimlikNoileAdresSorguKriteri>
                                <ns1:KimlikNo>%s</ns1:KimlikNo>
                            </ns1:KimlikNoileAdresSorguKriteri>
                        </ns2:kriterListesi>
                    </ns2:Sorgula>
            </env:Body>""" % tckn
        super(AdresSorgula, self).handle()


class AileBireySorgula(NVIService):
    """
    Aile Birey Sorgulama
    """

    HAS_CHANNEL = True

    def handle(self):
        tckn = self.request.payload['tckn']
        self.service_action = "/2011/01/01/AileListesiAraTCNoServis/Sorgula"
        self.service_xml_body = """
            <env:Body xmlns:env="http://www.w3.org/2003/05/soap-envelope" xmlns:ns1="http://kps.nvi.gov.tr/2011/01/01">
                <ns1:Sorgula>
                    <ns1:kriterListesi>
                        <ns1:AileListesiAraTCNoSorguKriteri>
                            <ns1:KimlikNo>%s</ns1:KimlikNo>
                        </ns1:AileListesiAraTCNoSorguKriteri>
                    </ns1:kriterListesi>
                </ns1:Sorgula>
            </env:Body>""" % tckn

        super(AileBireySorgula, self).handle()



# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP Okul Sorgula

Hitap üzerinden personelin okul bilgilerinin sorgulamasını yapar.

"""

from ulakbus.services.personel.hitap.hitap_sorgula import HITAPSorgula


class HizmetOkulGetir(HITAPSorgula):
    """
    HITAP Sorgulama servisinden kalıtılmış Okul Bilgisi Sorgulama servisi

    """
    HAS_CHANNEL = True

    def handle(self):
        """
        Servis çağrıldığında tetiklenen metod.

        Attributes:
            service_name (str): İlgili Hitap sorgu servisinin adı
            bean_name (str): Hitap'tan gelen bean nesnesinin adı
            service_dict (dict): Hitap servisinden gelen kayıtların alanları,
                    ``HizmetOkul`` modelinin alanlarıyla eşlenmektedir.
                    Servis tarafında gerekli olan alanlar listede tutulmaktadır.

        """

        self.service_name = 'HizmetOkulSorgula'
        self.bean_name = 'HizmetEgitimOkulServisBean'
        self.service_dict = {
            'fields': {
                'tckn': 'tckn',
                'kayit_no': 'kayitNo',
                'ogrenim_durumu': 'ogrenimDurumu',
                'mezuniyet_tarihi': 'mezuniyetTarihi',
                'okul_ad': 'okulAd',
                'bolum': 'bolum',
                'ogrenim_yeri': 'ogrenimYer',
                'denklik_tarihi': 'denklikTarihi',
                'denklik_okul': 'denklikOkul',
                'denklik_bolum': 'denklikBolum',
                'ogrenim_suresi': 'ogrenimSuresi',
                'hazirlik': 'hazirlik',
                'kurum_onay_tarihi': 'kurumOnayTarihi'
            },
            'date_filter': ['mezuniyet_tarihi', 'denklik_tarihi', 'kurum_onay_tarihi'],
            'required_fields': ['tckn']
        }
        super(HizmetOkulGetir, self).handle()



# -*-  coding: utf-8 -*-

from zengine.views.crud import CrudView, obj_filter
from zengine.forms import JsonForm
from zengine.forms import fields
from zengine.lib.translation import gettext as _, gettext_lazy
from ulakbus.models.personel import Personel, UcretsizIzin
from ulakbus.models.hitap.hitap import HizmetKayitlari
from datetime import timedelta, date


class UcretsizIzinIslemleri(CrudView):
    class Meta:
        # CrudViev icin kullanilacak temel Model
        model = 'UcretsizIzin'
        exclude = ['donus_tarihi', 'donus_tip']

        # ozel bir eylem listesi hazirlayacagiz. bu sebeple listeyi bosaltiyoruz.
        # kayit tipine bagli olarak ekleyecegimiz eylemleri .append() ile ekleyecegiz
        object_actions = [
            #     # {'fields': [0, ], 'cmd': 'show', 'mode': 'normal', 'show_as': 'link'},
        ]

    class ListForm(JsonForm):
        btn = fields.Button(gettext_lazy(u"Ücretsiz İzine Ayır"), next="izine_ayir", cmd="save")  # default

    class IzinForm(JsonForm):
        class Meta:
            include = ['tip', 'baslangic', 'bitis', 'onay_tarihi', 'personel']
            title = gettext_lazy(u"İzine Ayır")

        kaydet = fields.Button(gettext_lazy(u"Kaydet"), next="izine_ayir", cmd="izine_ayir")

    class DonusForm(JsonForm):
        class Meta:
            include = ['donus_tarihi', 'donus_tip']

        kaydet = fields.Button(gettext_lazy(u"Kaydet"), next="izin_donus", cmd="izin_donus")

    def goster(self):
        if 'id' in self.input:
            personel = Personel.objects.get(self.input['id'])

            ucretsiz_izinler = UcretsizIzin.objects.filter(personel=personel)
            ucretsiz_izinde = False

            for izin in ucretsiz_izinler:
                if izin.donus_tarihi == None or izin.donus_tarihi == "":
                    ucretsiz_izinde = True
                    self.current.task_data['izin_id'] = izin.key
                    break

            if ucretsiz_izinde:
                self.ListForm.btn = fields.Button(_(u"Ücretsiz İzin Dönüşü"), cmd="izin_donus",
                                                object_id=self.current.task_data['izin_id'])
            else:
                self.ListForm.btn = fields.Button(_(u"Ücretsiz İzine Ayır"), cmd="izine_ayir")
        else:
            pass

        self.list()

    def izine_ayir(self):
        self.form_out(self.IzinForm(self.object, current=self.current))

    def izin_donusu(self):
        self.object = UcretsizIzin.objects.get(self.current.task_data['izin_id'])
        self.form_out(self.DonusForm(self.object, current=self.current))

    def kontrol(self):
        self.set_form_data_to_object()
        if self.current.task_data['cmd'] == 'izine_ayir':
            if self.object.baslangic > self.object.bitis:
                self.current.task_data['cmd'] = 'izne_ayir'
            else:
                hitap_kaydi = HizmetKayitlari()
                personel = self.object.personel
                hitap_kaydi.personel = personel
                hitap_kaydi.tckn = personel.tckn
                hitap_kaydi.bitis_tarihi = self.object.baslangic
                hitap_kaydi.gorev = ".."
                hitap_kaydi.hizmet_sinifi = personel.hizmet_sinifi
                hitap_kaydi.unvan_kod = personel.kadro.unvan

                ## TODO: Sebep Kodları fixtures eklenecek
                hitap_kaydi.sebep_kod = 269
                hitap_kaydi.kurum_onay_tarihi = self.object.onay_tarihi
                hitap_kaydi.sync = 2
                hitap_kaydi.save()
                self.current.task_data['cmd'] = 'basarili'
                self.save()
        else:  ## cmd="izin_donus"
            hitap_kaydi = HizmetKayitlari()
            personel = self.object.personel
            hitap_kaydi.personel = personel
            hitap_kaydi.tckn = personel.tckn
            hitap_kaydi.baslama_tarihi = self.object.donus_tarihi
            hitap_kaydi.gorev = ".."
            hitap_kaydi.hizmet_sinifi = personel.hizmet_sinifi
            hitap_kaydi.unvan_kod = personel.kadro.unvan

            ## TODO: Sebep Kodları fixtures eklenecek, form içerisinden seçilecek
            hitap_kaydi.sebep_kod = 269
            hitap_kaydi.kurum_onay_tarihi = self.object.onay_tarihi
            hitap_kaydi.sync = 2
            hitap_kaydi.save()
            self.current.task_data['cmd'] = 'basarili'
            self.save()

    @obj_filter
    def donus_yapilmis_mi(self, izin, result):
        """
        Ücretsiz İzin Dönüşü yapılmış mı kontrol edilecek
        Eğer izin dönüşü yapılmışsa Sil olmayacak

        :param obj: Kadro instance
        :param result: liste ogesi satiri
        :return: liste ogesi
        """
        if izin.donus_tarihi == None or izin.donus_tarihi == "":
            result['actions'].append(
                    {'name': _(u'Sil'), 'cmd': 'delete', 'show_as': 'button'})
        return result



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
import base64
import os
from uuid import uuid4

from boto.s3.connection import S3Connection as s3

from boto.s3.key import Key
from pyoko.conf import settings


class S3FileManager(object):
    def __init__(self):
        self.conn = s3(aws_access_key_id=settings.S3_ACCESS_KEY,
                       aws_secret_access_key=settings.S3_SECRET_KEY,
                       proxy=settings.S3_PROXY_URL,
                       proxy_port=settings.S3_PROXY_PORT,
                       is_secure=False)

    def store_file(self, **kwargs):
        bucket = self.conn.get_bucket(settings.S3_BUCKET_NAME)
        content = kwargs['content']
        k = Key(bucket)
        filename = None
        if 'name' in kwargs:
            filename, extension = os.path.splitext(kwargs['name'])
            extension = extension.replace('.', '')
        if 'ext' in kwargs:
            extension = kwargs['ext']
        typ, ext = settings.ALLOWED_FILE_TYPES[extension]
        if "type" in kwargs:
            typ = kwargs['type']
        if 'random_name' in kwargs:
            filename = None
        k.key = "%s.%s" % (filename or uuid4().hex, ext)
        k.content_type = typ
        try:
            content = base64.decodestring(content.split('base64,')[1])
        except IndexError:
            content = base64.decodestring(content)

        k.set_contents_from_string(content)
        bucket.set_acl('public-read', k.key)
        return k.key

    @staticmethod
    def get_url(key):
        return "%s%s" % (settings.S3_PUBLIC_URL, key)



# -*-  coding: utf-8 -*-
# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
#
from ulakbus.lib.views import UlakbusView
from zengine.forms import JsonForm
from zengine.forms import fields
from zengine.lib.translation import gettext as _
from ulakbus.lib.common import parola_kontrolleri


class ParolaDegistir(UlakbusView):
    """
    Kullanıcıların parolalarını değiştirebilmelerini sağlar.
    """

    def yeni_parola_girisi(self):
        """
        Parola değişikliği ekranını oluşturur. Kullanıcı parolasını değiştirirken hata yaparsa
        (8 karakterden az parola seçimi gibi), hata kontrol edildikten sonra yine bu ekranda
        gösterilir.

        """
        if self.current.task_data.get('msg', None):
            self.mesaj_kutusu_goster(_(u'Parola Hatalı'))

        _form = JsonForm(current=self.current, title=_(u'Parola Değiştirme'))
        _form.help_text = _(
u"""Kendi güvenliğiniz ve sistem güvenliği için yeni oluşturacağınız parola:

* Türkçe karakter içermemeli,
* 8 karakterden büyük olmalı,
* En az bir küçük harf, bir büyük harf, bir sayı ve bir özel karakter içermeli,
* Eski şifrenizle aynı olmamalıdır,
* Özel karakterler = ()[]{}!@#$%^&*+=-§±_~/|"><\.,:;≤≥
* Örnek parola = Ulakbüs3\*""")

        _form.eski_parola = fields.String(_(u"Şu an kullandığınız parolanızı giriniz."),
                                          type="password")
        _form.yeni_parola = fields.String(_(u"Yeni parolanızı giriniz."), type="password")
        _form.yeni_parola_tekrar = fields.String(_(u"Yeni parolanızı tekrar giriniz."),
                                                 type="password")
        _form.degistir = fields.Button(_(u"Parola Değiştir"))
        self.form_out(_form)

    def yeni_parola_kontrol(self):
        """
        Girilen parolalar önce Cache'e atılır. Ardından parola kontrol methodlarına gönderilir.
        Eğer parola kontrol methodlarından geçerse parola kaydetme adımına gidilir. Eğer parola
        değişikliğinde bir hata olursa hata mesajı ile bir önceki adıma gidilip hata gösterilir
        ve parola değişikliğini yeniden denemesi istenir.

        """
        eski_parola = self.input['form']['eski_parola']
        yeni_parola = self.input['form']['yeni_parola']
        self.current.task_data['yeni_parola'] = yeni_parola
        yeni_parola_tekrar = self.input['form']['yeni_parola_tekrar']
        kullanici = self.current.user

        self.current.task_data['uygunluk'], self.current.task_data['msg'] = \
            parola_kontrolleri(yeni_parola, yeni_parola_tekrar, kullanici, eski_parola)

    def yeni_parola_kaydet(self):
        """
        Cache'e atılan parolalar güvenlik nedeniyle tekrardan parola kontrol methodlarına
        gönderilir. Kontrolden geçerse parola değiştirilir ve işlem başarılı mesajı yaratılır.
        Eğer kontrolden geçmezse ki bu beklenmeyen bir durumdur. Beklenmeyen hata adımına gidilir
        ve hata gösterilir.

        """
        self.current.user.set_password(self.current.task_data['yeni_parola'])
        self.current.user.save()
        self.current.task_data['islem'] = True
        self.current.task_data['msg'] = None
        self.current.task_data['islem_mesaji'] = _(u"""Parolanız başarıyla değiştirildi. Çıkış
                                                    yapıp yeni parolanızla giriş yapabilirsiniz""")



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

from pyoko import ListNode
from pyoko.exceptions import ObjectDoesNotExist
from ulakbus.lib.view_helpers import prepare_choices_for_model
from ulakbus.models import Sube, OgrenciDersi, DersKatilimi
from zengine import forms
from zengine.forms import fields
from zengine.views.crud import CrudView
from zengine.lib.translation import gettext as _, gettext_lazy

__author__ = 'Ali Riza Keles'


class DevamsizlikForm(forms.JsonForm):
    class Meta:
        inline_edit = ['katilim_durumu', 'aciklama']
        title = gettext_lazy(u"Ders Seçim Formu")

    class Ogrenciler(ListNode):
        ogrenci_no = fields.String(gettext_lazy(u'Öğrenci No'))
        ad_soyad = fields.String(gettext_lazy(u'Ad Soyad'))
        katilim_durumu = fields.Integer(gettext_lazy(u'Katılım Durumu'))
        aciklama = fields.String(gettext_lazy(u'Açıklama'))
        ogrenci_key = fields.String('ogrenci_key', hidden=True)
        sube_key = fields.String('ders_key', hidden=True)


class OnizlemeForm(forms.JsonForm):
    class Meta:
        title = gettext_lazy(u"Katılım Durumu Bilgileri Önizleme Ekranı")

    class Ogrenciler(ListNode):
        ogrenci_no = fields.String(gettext_lazy(u'Öğrenci No'))
        ad_soyad = fields.String(gettext_lazy(u'Ad Soyad'))
        katilim_durumu = fields.Integer(gettext_lazy(u'Katılım Durumu'))
        aciklama = fields.String(gettext_lazy(u'Açıklama'))
        ogrenci_key = fields.String('ogrenci_key', hidden=True)
        sube_key = fields.String('ders_key', hidden=True)


class KatilimDurumu(CrudView):
    """ Ders Katılım Durumu

    Okutmanların öğrenci devamsızlıklarını sisteme girebilmesini
    sağlayan workflowa ait metotları barındıran sınıftır.

    Bu iş akışı 5 adımdan oluşur.

    Sube seç:
    Okutmanın kayıtlı olduğu şubelerden biri seçilir.

    Katılım Durumu Formu:
    Şubeye kayıtlı öğrencilerden bir form oluşturur. Yerinde düzenleme
    ile öğrenci devamsızlıkları girilir.

    Kontrol:
    Önceki adımda forma girilen bilgilerin doğruluğunun kontrol edilmesi
    için bilgileri bir tablo şeklinde ekrana getirir.

    Kaydet:
    Doğruluğu onaylanan bilgileri kaydeder.

    Bilgi Ver:
    Yapılan işlemin başarıyla sonuçlandığı ekrana basar.




    """

    class Meta:
        model = "DersKatilimi"

    def sube_sec(self):
        """
        Okutmanın kayıtlı olduğu şubelerden biri seçilir.

        """
        _form = forms.JsonForm(current=self.current, title=_(u"Şube Seçiniz."))
        _form.sube = fields.Integer(
            _(u"Sube Seçiniz"),
            choices=prepare_choices_for_model(Sube, okutman_id=self.get_okutman_key)
        )
        _form.sec = fields.Button(_(u"Seç"))
        self.form_out(_form)

    def katilim_durumu(self):
        """
        Seçile şubeye ait öğrencilerden  bir form oluşturur. Yerinde düzenleme
        ile öğrencilerin devamsızlıkları girilir.

        """

        _form = DevamsizlikForm(current=self.current, title=_(u"Ders Katılımı Giriş Formu"))

        try:
            sube_key = self.current.input['form']['sube']
            self.current.task_data["sube_key"] = sube_key
        except KeyError:
            sube_key = self.current.task_data["sube_key"]

        ogrenci_dersleri = OgrenciDersi.objects.filter(sube_id=sube_key)
        for ogrenci_dersi in ogrenci_dersleri:
            try:
                derse_katilim = DersKatilimi.objects.get(sube_id=sube_key,
                                                         ogrenci=ogrenci_dersi.ogrenci)
                katilim_durumu = derse_katilim.katilim_durumu
                aciklama = derse_katilim.aciklama
            except ObjectDoesNotExist:
                katilim_durumu = ""
                aciklama = ""

            _form.Ogrenciler(
                ad_soyad='%s %s' % (ogrenci_dersi.ogrenci.ad, ogrenci_dersi.ogrenci.soyad),
                ogrenci_no=ogrenci_dersi.ogrenci_program.ogrenci_no,
                katilim_durumu=katilim_durumu, ogrenci_key=ogrenci_dersi.ogrenci.key,
                sube_key=ogrenci_dersi.sube.key, aciklama=aciklama
            )

        _form.onizleme = fields.Button(_(u"Önizleme"), cmd="kontrol")
        self.form_out(_form)
        self.current.output["meta"]["allow_actions"] = False

    def kontrol(self):
        """
        Önceki adımda forma girilen bilgilerin doğruluğunun
        kontrol edilmesi için bilgileri bir tablo şeklinde ekrana
        getirir.

        """

        ogrenci_katilim_durumlari = self.current.input['form']['Ogrenciler']
        _form = OnizlemeForm(current=self.current)
        _form.duzenle = fields.Button(_(u"Geri Dön ve Düzenle"), flow="katilim_durumu")
        for katilim_durumu in ogrenci_katilim_durumlari:
            _form.Ogrenciler(
                ad_soyad=katilim_durumu['ad_soyad'],
                ogrenci_no=katilim_durumu['ogrenci_no'],
                katilim_durumu=katilim_durumu['katilim_durumu'],
                ogrenci_key=katilim_durumu['ogrenci_key'],
                sube_key=katilim_durumu['sube_key'], aciklama=katilim_durumu['aciklama']
            )
        _form.kaydet = fields.Button(_(u"Kaydet"))
        self.current.output["meta"]["allow_actions"] = False
        self.form_out(_form)

    def kaydet(self):
        """
        Doğruluğu onaylanan bilgileri kaydeder.

        """
        for katilim in self.current.input['form']['Ogrenciler']:
            ders_katilimi, is_new = DersKatilimi.objects.get_or_create(
                ogrenci_id=katilim['ogrenci_key'], sube_id=katilim['sube_key'])
            ders_katilimi.katilim_durumu = katilim['katilim_durumu']
            ders_katilimi.aciklama = katilim['aciklama']
            ders_katilimi.save()

    def bilgi_ver(self):
        """
        Yapılan işlemin başarıyla sonuçlandığı gösterir.

        """
        sube = Sube.objects.get(self.current.task_data['sube_key'])

        self.current.output['msgbox'] = {
            'type': 'info', "title": _(u'Devamsızlıklar Kaydedildi'),
            "msg": _(u'%s dersine ait öğrencilerin katılım bilgileri \
                           başarıyla kaydedildi') % sube.ders.ad}

    @property
    def get_okutman_key(self):
        """
        Harici okutman ve okutman kayıt key'lerinin ayrımını sağlayan metot.

        Returns:
            Okutman nesnesinin keyini

        """
        u = self.current.user
        return u.personel.okutman.key if u.personel.key else u.harici_okutman.okutman.key



# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

from zengine.messaging.model import Message, Channel, Subscriber
from .personel import *
from .auth import *
from .ogrenci import *
from .hitap.hitap import *
from .buildings_rooms import *
from .form import *
from .ders_sinav_programi import *
from zengine.models.workflow_manager import DiagramXML, WFInstance, Task, BPMNWorkflow
from .zato import *



# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP Açık Süre Sil

Hitap'da personelin açık süre bilgilerinin silinmesi sağlayan class.

"""

from ulakbus.services.personel.hitap.hitap_sil import HITAPSil


class HizmetAcikSureSil(HITAPSil):
    """
    HITAP Silme servisinden kalıtılmış Hizmet Açık Süre Bilgisi Silme servisi

    """
    HAS_CHANNEL = True

    def handle(self):
        """Servis çağrıldığında tetiklenen metod.

        Attributes:
            service_name (str): İlgili Hitap sorgu servisinin adı
            service_dict (dict): Request yolula gelen kayıtlar,
                    HizmetAcikSureDelete servisinin alanlarıyla eşlenmektedir.
                    Servis tarafında gerekli olan alanlar listede tutulmaktadır.

        """
        self.service_name = 'HizmetAcikSureDelete'

        self.service_dict['fields']['tckn'] = self.request.payload.get('tckn', '')
        self.service_dict['fields']['kayitNo'] = self.request.payload.get('kayit_no', '')
        self.service_dict['required_fields'] = ['tckn', 'kayit_no']
        super(HizmetAcikSureSil, self).handle()



# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP Açık Süre Güncelle

Hitap'a personelin açık süre bilgilerinin güncellemesini yapar.

"""

from ulakbus.services.personel.hitap.hitap_guncelle import HITAPGuncelle


class HizmetAcikSureGuncelle(HITAPGuncelle):
    """
    HITAP Güncelleme servisinden kalıtılmış Hizmet Açık Süre Bilgisi Güncelleme servisi

    """
    HAS_CHANNEL = True

    def handle(self):
        """Servis çağrıldığında tetiklenen metod.

        Attributes:
            service_name (str): İlgili Hitap sorgu servisinin adı
            service_dict (dict): Request ile gelen kayıtlar,
                    HizmetAcikSureUpdate servisinin alanlarıyla eşlenmektedir.
                    Filtreden geçecek tarih alanları ve gerekli alanlar listede tutulmaktadır.
        """

        self.service_name = 'HizmetAcikSureUpdate'
        hizmet_acik_sure = HizmetAcikSure.objects.get(key)
        self.service_dict = {
            'fields': {
                'kayitNo': self.request.payload.get('kayit_no', ''),
                'tckn': self.request.payload.get('tckn', ''),
                'acikSekil': self.request.payload.get('acik_sekil', ''),
                'iadeSekil': self.request.payload.get('iade_sekil', ''),
                'hizmetDurum': self.request.payload.get('hizmet_durum', ''),
                'husus': self.request.payload.get('husus', ''),
                'acigaAlinmaTarih': self.request.payload.get('aciga_alinma_tarih', ''),
                'goreveSonTarih': self.request.payload.get('goreve_son_tarih', ''),
                'goreveIadeIstemTarih': self.request.payload.get('goreve_iade_istem_tarih', ''),
                'goreveIadeTarih': self.request.payload.get('goreve_iade_tarih', ''),
                'acikAylikBasTarihi': self.request.payload.get('acik_aylik_bas_tarih', ''),
                'acikAylikBitTarihi': self.request.payload.get('acik_aylik_bit_tarih', ''),
                'gorevSonAylikBasTarihi': self.request.payload.get('goreve_son_aylik_bas_tarih',
                                                                   ''),
                'gorevSonAylikBitTarihi': self.request.payload.get('goreve_son_aylik_bit_tarih',
                                                                   ''),
                'SYonetimKaldTarih': self.request.payload.get('s_yonetim_kald_tarih', ''),
                'aciktanAtanmaTarih': self.request.payload.get('aciktan_atanma_tarih', ''),
                'kurumOnayTarihi': self.request.payload.get('kurum_onay_tarihi', '')
            },
            'date_filter': ['acigaAlinmaTarih', 'goreveSonTarih', 'goreveIadeIstemTarih',
                            'goreveIadeTarih', 'acikAylikBasTarihi', 'acikAylikBitTarihi',
                            'gorevSonAylikBasTarihi', 'gorevSonAylikBitTarihi', 'SYonetimKaldTarih',
                            'aciktanAtanmaTarih', 'kurumOnayTarihi'],
            'required_fields': ['kayitNo', 'tckn', 'acikSekil', 'durum', 'hizmetDurum', 'husus',
                                'kurumOnayTarihi']
        }
        super(HizmetAcikSureGuncelle, self).handle()
